// browsewindow.cpp
//
/****************************************************************************
   Copyright (C) 2004, David Henry. All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*****************************************************************************/

#include "../config.h"

#include "addserverdialog.h"
#include "browsewindow.h"
#include "gsms.h"
#include "gsmsXMLdoc.h"
#include "gspyquery.h"
#include "filterdialog.h"
#include "settingsdialog.h"
#include "retrydialog.h"
#include "searchdialog.h"
#include "serverlistview.h"
#include "buddylist.h"
#include "httpquery.h"
#include "parsehttpquery.h"
#include "newgspyquery.h"
#include "gspymaster.h"

#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qstatusbar.h>
#include <qapplication.h>
#include <qlistview.h>
#include <qmessagebox.h>
#include <qprocess.h>
#include <qprogressbar.h>
#include <qlabel.h>
#include <qclipboard.h>
#include <qlineedit.h>
#include <qsettings.h>
#include <qwidgetstack.h>
#include <qvbox.h>
#include <qheader.h>
#include <qinputdialog.h>
#include <qaction.h>

#include "../xpms/connect.xpm"
#include "../xpms/update.xpm"
#include "../xpms/refresh.xpm"
#include "../xpms/filters.xpm"
#include "../xpms/search.xpm"
#include "../xpms/settings.xpm"
#include "../xpms/stop.xpm"
#include "../xpms/favourites.xpm"
#include "../xpms/buddies.xpm"
#include "../xpms/aamsl.xpm"
#include "../xpms/andromeda.xpm"
#include "../xpms/gamespy.xpm"

const QString SETTINGS_DIR = "/.aabrowse/";
const QString APP_KEY = "/aabrowse/";

BrowseWindow::BrowseWindow()
   : QMainWindow( 0, "aabrowse main window", WDestructiveClose )
{
   refreshAllAction = new QAction ( QPixmap ( refresh_xpm ), "Refresh &all", 
      CTRL+Key_A, this );
   refreshSelAction = new QAction ( "Refresh selected", 0, this );
   refreshCurAction = new QAction ( "&Refresh current", CTRL+Key_R, this );
   delAllAction = new QAction ( "Delete all", 0, this );
   delTimeAction = new QAction ( "Delete timed out", 0, this );
   delCurAction = new QAction ( "Delete current", 0, Key_Delete, this );
   connectAction = new QAction ( QPixmap ( connect_xpm ), "Connect",
      Key_Return, this );
   copyAction = new QAction ( "&Copy address", CTRL+Key_C, this );
   addServerAction = new QAction ( "Add new server", 0, this );
   addFavsAction = new QAction ( "Add to favourites", 0, this );
   stopAction = new QAction ( QPixmap ( stop_xpm ), "Stop", 0, this );
   updateAction = new QAction ( QPixmap ( update_xpm ), "&Update", CTRL+Key_U,
      this );
   updateAction->setEnabled ( FALSE );   
   filterAction = new QAction ( QPixmap ( filters_xpm ), "Filters", 0, this );
   filterAction->setMenuText ( "Master server filters" );
   settingsAction = new QAction ( QPixmap ( settings_xpm ), "Settings", 0,
      this );
   searchAction = new QAction ( QPixmap ( search_xpm ), "Search", 0,
      this );

   // Master server selector action group
   QActionGroup *selectGroup = new QActionGroup ( this, "select", TRUE );
   gspyAction = new QAction ( QPixmap ( gamespy_xpm ), "GameSpy", 0,
      selectGroup );
   androAction = new QAction ( QPixmap ( andromeda_xpm ), "Andromeda",
      0, selectGroup );
   aamslAction = new QAction ( QPixmap ( aamsl_xpm ), "AAMSL.NET", 0,
      selectGroup );
   selectGroup->setText ( "Select server" );
   selectGroup->setUsesDropDown ( TRUE );
   gspyAction->setToggleAction ( TRUE );
   androAction->setToggleAction ( TRUE );
   aamslAction->setToggleAction ( TRUE );

   // Load filter action group
   loadFilter = new QPopupMenu ( this );
   l1 = loadFilter->insertItem ( "Set 1" );
   l2 = loadFilter->insertItem ( "Set 2" );
   l3 = loadFilter->insertItem ( "Set 3" );
   l4 = loadFilter->insertItem ( "Set 4" );
   l0 = loadFilter->insertItem ( "Enable All" );
   // Save filter action group
   saveFilter = new QPopupMenu ( this );
   s1 = saveFilter->insertItem ( "Set 1" );
   s2 = saveFilter->insertItem ( "Set 2" );
   s3 = saveFilter->insertItem ( "Set 3" );
   s4 = saveFilter->insertItem ( "Set 4" );
   // Rename filter action group
   renameFilter = new QPopupMenu ( this );
   r1 = renameFilter->insertItem ( "Set 1" );
   r2 = renameFilter->insertItem ( "Set 2" );
   r3 = renameFilter->insertItem ( "Set 3" );
   r4 = renameFilter->insertItem ( "Set 4" );
   
   connect ( loadFilter, SIGNAL ( activated (int)), this,
      SLOT ( loadFilters (int)));
   connect ( saveFilter, SIGNAL ( activated (int)), this,
      SLOT ( saveFilters (int)));
   connect ( renameFilter, SIGNAL ( activated (int)), this,
      SLOT ( renameFilters(int)));
   connect ( selectGroup, SIGNAL ( selected ( QAction * )), this,
      SLOT ( changeMaster ( QAction * )));
   connect ( refreshAllAction, SIGNAL ( activated()), this,
      SLOT ( refreshList()));
   connect ( refreshSelAction, SIGNAL ( activated()), this,
      SLOT ( refreshSelected()));
   connect ( refreshCurAction, SIGNAL ( activated()), this,
      SLOT ( refreshSingle()));
   connect ( delAllAction, SIGNAL ( activated()), this,
      SLOT ( clearServerList()));
   connect ( delTimeAction, SIGNAL ( activated()), this,
      SLOT ( removeTimedoutServers()));
   connect ( delCurAction, SIGNAL ( activated()), this,
      SLOT ( deleteServer()));
   connect ( connectAction, SIGNAL ( activated()), this,
      SLOT ( attemptConnect()));
   connect ( copyAction, SIGNAL ( activated()), this,
      SLOT ( copyServerAddr()));
   connect ( addServerAction, SIGNAL ( activated()), this,
      SLOT ( getNewServer()));
   connect ( addFavsAction, SIGNAL ( activated()), this,
      SLOT ( moveToFavs()));
   connect ( stopAction, SIGNAL ( activated()), this,
      SLOT ( stop()));
   connect ( updateAction, SIGNAL ( activated()), this,
      SLOT ( updateAndromedaList()));
   connect ( filterAction, SIGNAL ( activated()), this,
      SLOT ( filterSettings()));
   connect ( settingsAction, SIGNAL ( activated()), this,
      SLOT ( pref()));
   connect ( searchAction, SIGNAL ( activated()), this,
      SLOT ( playerSearch()));

   // Create toolbars
   QToolBar *connectBar = new QToolBar ( this );
   connectBar->setLabel ( "Connect toolbar" );
   QToolBar *serverBar = new QToolBar ( this );
   serverBar->setLabel ( "Refresh toolbar" );
   QToolBar *settingsBar = new QToolBar ( this );
   settingsBar->setLabel ( "Settings toolbar" );
   QToolBar *masterBar = new QToolBar ( this );
   masterBar->setLabel ( "Master server" );
   
   connectAction->addTo ( connectBar );
   updateAction->addTo ( serverBar );
   refreshAllAction->addTo ( serverBar );
   stopAction->addTo ( serverBar );
   selectGroup->addTo ( masterBar );

   settingsAction->addTo ( settingsBar );
   filterAction->addTo ( settingsBar );
   searchAction->addTo ( settingsBar );
   
   QPopupMenu *fileMenu = new QPopupMenu ( this );
   menuBar()->insertItem ( "&File", fileMenu );
   searchAction->addTo ( fileMenu );
   settingsAction->addTo ( fileMenu );
   fileMenu->insertSeparator();
   fileMenu->insertItem( "&Quit", qApp, SLOT(closeAllWindows()), CTRL+Key_Q );

   QPopupMenu *masterMenu = new QPopupMenu ( this );
   menuBar()->insertItem ( "&Master", masterMenu );
   updateAction->addTo ( masterMenu );
   selectGroup->addTo ( masterMenu );

   QPopupMenu *serverMenu = new QPopupMenu ( this );
   menuBar()->insertItem ( "&Server", serverMenu );
   refreshAllAction->addTo ( serverMenu );
   refreshSelAction->addTo ( serverMenu );
   refreshCurAction->addTo ( serverMenu );

   serverMenu->insertSeparator ();

   delAllAction->addTo ( serverMenu );
   delTimeAction->addTo ( serverMenu );
   delCurAction->addTo ( serverMenu );

   serverMenu->insertSeparator ();

   connectAction->addTo ( serverMenu );
   copyAction->addTo ( serverMenu );

   serverMenu->insertSeparator ();
   addServerAction->addTo ( serverMenu );
   addFavsAction->addTo ( serverMenu );

   QPopupMenu *filterMenu = new QPopupMenu ( this );
   menuBar()->insertItem ( "&Filters", filterMenu );
   filterAction->addTo ( filterMenu );
   filterMenu->insertItem ( "Load filters", loadFilter );
   filterMenu->insertItem ( "Save filters", saveFilter );
   filterMenu->insertItem ( "Rename filters", renameFilter );

   QPopupMenu * help = new QPopupMenu( this );
   menuBar()->insertItem( "&Help", help );

   help->insertItem( "&About", this, SLOT(about()), Key_F1 );

   // Create status bar
   statusLabel = new QLabel ( statusBar());
   statusBar()->addWidget ( statusLabel, 5, TRUE );

   statsLabel = new QLabel ( statusBar());
   statusBar()->addWidget ( statsLabel, 2, TRUE );

   pingProgress = new QProgressBar ( statusBar());
   pingProgress->setTotalSteps ( 0 );
   pingProgress->setProgress ( 0 );
   statusBar()->addWidget ( pingProgress, 2, TRUE );
   
   // Create main server view
   QSplitter *mainWindow = new QSplitter ( this );
   setCentralWidget ( mainWindow );

   QVBox *sideBarBox = new QVBox ( mainWindow );

   // Create list for selected each serverlist or the buddylist
   selectList = new QListView ( sideBarBox );
   selectList->header()->hide();
   selectList->setRootIsDecorated ( TRUE );
   selectList->addColumn ( "Servers" );
   selectList->setSorting ( -1 );   // Disable sorting
   setupSelectList();

   // Create box for entering server password
   (void) new QFrame ( sideBarBox );

   (void) new QLabel ( "Password:", sideBarBox);

   passwordEdit = new QLineEdit ( sideBarBox);
   passwordEdit->setEchoMode ( QLineEdit::Password );

   setStopped ( TRUE );   // Is true when refreshing a server
   updating = FALSE;

   connect ( selectList, SIGNAL ( clicked ( QListViewItem * )), this, 
      SLOT ( changeServerView( QListViewItem *)));
   connect ( selectList, SIGNAL ( contextMenuRequested ( QListViewItem *,
      const QPoint&, int )), this, SLOT ( selectListRMB ( QListViewItem *,
      const QPoint&, int )));
   
   // Create stack of 2 server widgets and a buddylist
   servers = new QWidgetStack ( mainWindow );
   serversFavs = new ServerListView ( servers );
   serversAndro = new ServerListView ( servers );
   buddyList = new BuddyList ( servers );
   serversFavs->setBuddyList ( buddyList );
   serversAndro->setBuddyList ( buddyList );
   servers->addWidget ( serversFavs, 0 );
   servers->addWidget ( serversAndro, 1 );
   servers->addWidget ( buddyList, 2 );
   servers->raiseWidget ( 0 );

   currentServerList = serversFavs;

   // Connect serverlist mouse clicks to SLOTS
   connect ( serversFavs, SIGNAL ( serverRightClick ( QListViewItem *,
      const QPoint&, int )), this, SLOT ( serverRMB ( QListViewItem *,
      const QPoint&, int )));
   connect ( serversAndro, SIGNAL ( serverRightClick ( QListViewItem *,
      const QPoint&, int )), this, SLOT ( serverRMB ( QListViewItem *,
      const QPoint&, int )));
			      
   connect ( serversFavs, SIGNAL ( playerRightClick ( QListViewItem *,
      const QPoint&, int )), this, SLOT ( playerRMB ( QListViewItem *,
      const QPoint&, int )));
   connect ( serversAndro, SIGNAL ( playerRightClick ( QListViewItem *,
      const QPoint&, int )), this, SLOT ( playerRMB ( QListViewItem *,
      const QPoint&, int )));

   connect ( serversFavs, SIGNAL ( serverDoubleClicked ( QListViewItem *,
      const QPoint &, int )), SLOT ( attemptConnect ()));
   connect ( serversAndro, SIGNAL ( serverDoubleClicked ( QListViewItem *,
      const QPoint &, int )), SLOT ( attemptConnect ()));

   connect ( serversFavs, SIGNAL ( serverClicked ( QListViewItem * )),
      SLOT ( serverLMB ( QListViewItem * )));
   connect ( serversAndro, SIGNAL ( serverClicked ( QListViewItem * )),
      SLOT ( serverLMB ( QListViewItem * )));

   connect ( buddyList, SIGNAL ( join ( QString )), this,
      SLOT ( connectToGameServer ( QString )));
   connect ( buddyList, SIGNAL ( raiseServer ( QString, QString )), this,
      SLOT ( findServer ( QString, QString )));

   connect ( qApp, SIGNAL ( aboutToQuit ()), SLOT ( saveSettings ()));

   queryList.resize ( 101 );
   loadStartupSettings();
   checkActions();
}

BrowseWindow::~BrowseWindow()
{
}

void BrowseWindow::about()
{
   QMessageBox::about( this, "About AABrowse",
		       "<b>AABrowse " VERSION "</b><br><br>AABrowse is a server browser for Americas Army.<br><br>http://sourceforge.net/projects/aabrowse/" );
}

//-------------------------------------------------------------------
//  SLOT called when clicked on select list. Raises appropriate widget
//  to the top of the stack.
//-------------------------------------------------------------------
void BrowseWindow::changeServerView( QListViewItem *newServerList )
{
   if ( !newServerList )
      return;
   checkActions();
   if ( newServerList->text ( 0 ) == "Buddy List" )
   {
      servers->raiseWidget ( 2 );
      updateBuddyCount ( );
      return;
   }
   if ( newServerList->text ( 0 ) == "Favourites" )
   {
      servers->raiseWidget ( 0 );
      updatePlayerServerCount ( );
      return;
   }
   if ( newServerList->text ( 0 ) != "Master Server" )
      filterList ();
   servers->raiseWidget ( 1 );
   
   updatePlayerServerCount ( );
   return;
}

void BrowseWindow::checkActions()
{
   refreshSelAction->setEnabled ( FALSE );
   refreshCurAction->setEnabled ( FALSE );
   delCurAction->setEnabled ( FALSE );
   copyAction->setEnabled ( FALSE );
   connectAction->setEnabled ( FALSE );
   addServerAction->setEnabled ( FALSE );
   addFavsAction->setEnabled ( FALSE );
   if ( selectList->currentItem()->text(0) == "Buddy List" )
   {
      updateAction->setEnabled ( FALSE );
      refreshAllAction->setEnabled ( FALSE );
      delAllAction->setEnabled ( FALSE );
      delTimeAction->setEnabled ( FALSE );
      stopAction->setEnabled ( FALSE );
   }
   else
   {
      refreshAllAction->setEnabled ( TRUE );
      delTimeAction->setEnabled ( TRUE );
      stopAction->setEnabled ( TRUE );
      if ( selectList->currentItem()->text(0) == "Favourites" )
      {
	 updateAction->setEnabled ( FALSE );
	 delAllAction->setEnabled ( FALSE );
	 addServerAction->setEnabled ( TRUE );
      }
      else
      {
         updateAction->setEnabled ( TRUE );
	 delAllAction->setEnabled ( TRUE );
      }
   }
}

void BrowseWindow::changeMaster ( QAction *a )
{
   QSettings settings;
   settings.insertSearchPath ( QSettings::Unix, QDir::homeDirPath() +
                               SETTINGS_DIR );
   QString masterText = a->text();
   QPixmap masterPixmap;
   if ( masterText == "GameSpy" )
   {
      masterType = 1;
      masterPixmap = QPixmap ( gamespy_xpm );
   }
   else if ( masterText == "Andromeda" )
   {
      masterType = 2;
      masterPixmap = QPixmap ( andromeda_xpm );
   }
   else
   {
      masterType = 3;
      masterPixmap = QPixmap ( aamsl_xpm );
   }
   andromeda->setPixmap ( 0, masterPixmap );
   settings.writeEntry ( APP_KEY + "Settings/" + "master type", masterType );
}

//-------------------------------------------------------------------
//  Refreshes main serverList from SCI master server
//-------------------------------------------------------------------
void BrowseWindow::updateAndromedaList()
{
   if ( updating )
      return;
   updating = TRUE;
   if ( masterType == 1 )
   {
      statusLabel->setText ( "Connecting to GameSpy master server..." );
      masterGspy = new GSpyMaster( updateTimeout );
      masterGspy->setServerList ( serversAndro );
      masterGspy->connectToHost ( gspyMaster, 28900 );
      connect ( masterGspy, SIGNAL ( queryFinished()), this,
         SLOT ( finishedMaster ()));
      connect ( masterGspy, SIGNAL ( serverTimeout()), this,
         SLOT ( masterTimeout()));
   }
   else if ( masterType == 2 )
   {
      //   Download server list via andromeda
      statusLabel->setText ( "Connecting to SCI master server..." );
      masterGsms = new Gsms ( updateTimeout );
      masterGsms->setPacketType ( "COUNT" );
      masterGsms->connectToHost ( androMaster, 14200 );

      connect ( masterGsms, SIGNAL ( numberOfServers ( int )),
         SLOT ( numServers( int )) );
      connect ( masterGsms, SIGNAL ( serverList ( QString )),
         SLOT ( parseServerList ( QString )));
      connect ( masterGsms, SIGNAL ( bytesRead ( int )),
         SLOT ( updateBytes ( int )));
      connect ( masterGsms, SIGNAL ( serverTimeout ()),
         SLOT ( masterTimeout()));
   }
   else if ( masterType == 3 )
   {
      //   Download server list via http
      masterQuery = new HttpQuery ();
      statusLabel->setText ( "Connecting to AASML.NET..." );
      connect ( masterQuery, SIGNAL ( bytesRead ( int )), this,
         SLOT ( updateBytes ( int )));
      connect ( masterQuery, SIGNAL ( transferFinished ( bool )), this,
         SLOT ( parseHttpQuery ( bool )));
   }
   pingProgress->setTotalSteps ( 0 );
}

//-------------------------------------------------------------------
//  Clears current serverList and searches xml for beginning of server info.
//-------------------------------------------------------------------
void BrowseWindow::parseServerList ( QString list )
{
   delete masterGsms;
   updating = FALSE;
   pingProgress->setProgress ( pingProgress->totalSteps());

   GsmsXMLDoc doc ( "server list" );
   doc.setContent ( list );
   if ( !doc.setContent ( list ))
      statusLabel->setText ( "Could not pass serverlist. Please try again" );

   doc.parseServerList ( serversAndro );
   filterList();
   statusLabel->setText ( "Done" );
   if ( refreshMaster )
      refreshList();
   pingProgress->setProgress ( 0 );
   pingProgress->setTotalSteps ( 0 );
}

//-------------------------------------------------------------------
//  Display number of servers found and guess size of serverlist
//  Andromeda only
//------------------------------------------------------------------- 
void BrowseWindow::numServers ( int num )
{
   QString numString = QString::number ( num );
   numString = numString + " servers found";
   statusLabel->setText ( numString );

   // Guess number of bytes to download based on number of servers
   pingProgress->setTotalSteps ( (int) (num / 1.32 ));
   pingProgress->setProgress ( 0 );
}

//-------------------------------------------------------------------
//  Update statusbar with number of bytes of serverList currently downloaded
//-------------------------------------------------------------------
void BrowseWindow::updateBytes( int bytes )
{ 
   bytes = bytes / 1000;
   QString byteString = QString::number ( bytes );
   statusLabel->setText ( byteString + "KB read" );
   if ( masterType == 2 )
      pingProgress->setProgress ( bytes );
}

//-------------------------------------------------------------------
//  Parse the returned PHP file and update serverlist
//-------------------------------------------------------------------
void BrowseWindow::parseHttpQuery ( bool finished )
{
   delete masterQuery;
   updating = FALSE;
   pingProgress->reset();
   if ( finished )
   {
      statusLabel->setText ( "Done" );
      ParseHttpQuery *httpList = new ParseHttpQuery ();
      httpList->parseList ( serversAndro, "aamsl.php" );
      filterList();
      if ( refreshMaster )
         refreshList();
      delete httpList;
   }
   else
      statusLabel->setText ( "Failed to download serverlist" );
}

void BrowseWindow::finishedMaster ( )
{
   delete masterGspy;
   updating = FALSE;
   pingProgress->reset();
   filterList();
   if ( refreshMaster )
      refreshList();
   else
      statusLabel->setText ( "Done" );
}

void BrowseWindow::masterTimeout()
{
   if ( masterType == 1 )
      delete masterGspy;
   else if ( masterType == 2 )
      delete masterGsms;
   else if ( masterType == 3 )
      delete masterQuery;

   updating = FALSE;
   pingProgress->reset();
   statusLabel->setText ( "Timeout" );
}

//-------------------------------------------------------------------
//  Displays filter dialog. Filters are serverside
//-------------------------------------------------------------------
void BrowseWindow::filterSettings()
{
   FilterDialog *filters = new FilterDialog ( this, "filters", FALSE,
      Qt::WDestructiveClose );
   filters->setCaption ( "Master server filters" );
   filters->show();
}

//-------------------------------------------------------------------
//  Display settings dialog
//-------------------------------------------------------------------
void BrowseWindow::pref()
{  
   SettingsDialog *settings = new SettingsDialog ( this, "settings", FALSE, 
      Qt::WDestructiveClose );
   settings->setCaption ( "Settings" );
   settings->show();
   connect ( settings, SIGNAL ( settingsChanged()), SLOT ( reloadSettings()));
}  

void BrowseWindow::playerSearch()
{
   SearchDialog *search = new SearchDialog ( this, "player search", FALSE,
      Qt::WDestructiveClose );
   connect ( search, SIGNAL ( raiseServer ( QString, QString )), this,
      SLOT ( findServer ( QString, QString )));
   search->setListViews ( serversFavs, serversAndro );
   search->setBuddyList ( buddyList );
   search->setCaption ( "Player search" );
   search->show();
}

void BrowseWindow::stop ()
{
   setStopped ( TRUE );
   statusLabel->setText ( "Stopped" );
   pingProgress->reset();
}

void BrowseWindow::setStopped ( bool value )
{
   stopped = value;
}

bool BrowseWindow::isStopped ()
{
   return stopped;
}

//-------------------------------------------------------------------
//  Search for the server, raise it if its found
//-------------------------------------------------------------------
void BrowseWindow::findServer ( QString address, QString playerName )
{
   if ( serversFavs->showServer ( address, playerName ))
   {
      selectList->setSelected ( favourites, TRUE );
      servers->raiseWidget ( 0 );
      updateAction->setEnabled ( FALSE );
      delAllAction->setEnabled ( FALSE );
      addFavsAction->setEnabled ( FALSE );
      addServerAction->setEnabled ( TRUE );
      refreshAllAction->setEnabled ( TRUE );
      delTimeAction->setEnabled ( TRUE );
      stopAction->setEnabled ( TRUE );
   }
   else if ( serversAndro->showServer ( address, playerName ))
   {
      selectList->setSelected ( andromeda, TRUE );
      servers->raiseWidget ( 1 );
      updateAction->setEnabled ( TRUE );
      delAllAction->setEnabled ( TRUE );
      addServerAction->setEnabled ( FALSE );
      addFavsAction->setEnabled ( TRUE );
      refreshAllAction->setEnabled ( TRUE );
      delTimeAction->setEnabled ( TRUE );
      stopAction->setEnabled ( TRUE );
   }
   statusLabel->setText ( "Server not found" );
}

//-------------------------------------------------------------------
//  Refresh each server in the serverList
//-------------------------------------------------------------------
void BrowseWindow::refreshList ( )
{
   currentServerList = ( ServerListView* )servers->visibleWidget();
   setStopped ( FALSE );
   numSockets = 0;
   
   // Clear any previous addresses
   address.clear();
   address = currentServerList->getServerAddressList();
   pingProgress->setTotalSteps ( address.count());
   pingProgress->setProgress ( 0 );
   statusLabel->setText ( "Refreshing 0 / " + QString::number 
      ( pingProgress->totalSteps()) + " servers" );
   while ( address.begin() != address.end() && numSockets < maxSockets )
   {
      refreshNextServer();
      numSockets++;
   }
}

//-------------------------------------------------------------------
//  Refresh all selected items in a serverlist
//-------------------------------------------------------------------
void BrowseWindow::refreshSelected ()
{
   currentServerList = ( ServerListView * )servers->visibleWidget();
   setStopped ( FALSE );
   numSockets = 0;

   address.clear();
   address = currentServerList->getSelectedAddressList();
   if ( address.begin() == address.end() )
      return;
   pingProgress->setTotalSteps ( address.count());
   pingProgress->setProgress ( 0 );
   statusLabel->setText ( "Refreshing 0 / " + QString::number
      ( pingProgress->totalSteps()) + " servers" );
   while ( address.begin() != address.end() && numSockets < maxSockets )
   {
      refreshNextServer();
      numSockets++;
   }
}

//-------------------------------------------------------------------
//  Uses the gamespy protocol to query each server
//-------------------------------------------------------------------
void BrowseWindow::refreshNextServer ()
{
   // Only keep refreshing if we aren't stopped
   if ( !isStopped() )
   {
      QString serverAddress = address.first();
      if ( useNewGspy )
         queryList.insert ( serverAddress, new NewGSpyQuery());
      else 
         queryList.insert ( serverAddress, new GSpyQuery());
      queryList[serverAddress]->setAddress ( serverAddress );
      queryList[serverAddress]->setTimeout ( timeout );
      queryList[serverAddress]->setServerList ( currentServerList );
      queryList[serverAddress]->start();
      queryList[serverAddress]->sendQuery ( serverAddress );
      connect ( queryList[serverAddress], SIGNAL ( finished ( QString )),
                SLOT ( finishedQuery ( QString )));
      connect ( queryList[serverAddress], SIGNAL ( time ( int, QString )),
                SLOT ( refreshPing ( int, QString )));
      connect ( queryList[serverAddress], SIGNAL ( timeoutError ( QString )),
                SLOT ( refreshTimeout ( QString )));
      address.remove ( address.begin());
   }
}

//-------------------------------------------------------------------
//  SLOT called when a query returns a ping time. Updated servers time
//-------------------------------------------------------------------
void BrowseWindow::refreshPing ( int time, QString serverAddress )
{
   if ( isStopped() )
      return;
   currentServerList->setCurrentServer ( serverAddress );
   currentServerList->setServerPing ( time );
   pingProgress->setProgress ( pingProgress->progress() + 1 );
   statusLabel->setText ( "Refreshing " + QString::number 
      ( pingProgress->progress()) + " / " + QString::number 
      ( pingProgress->totalSteps()) + " servers" );
}

//-------------------------------------------------------------------
//  Timeout on query has expired
//-------------------------------------------------------------------
void BrowseWindow::refreshTimeout ( QString serverAddress )
{
   delete queryList[serverAddress];
   queryList.remove ( serverAddress );
   if ( isStopped() )
      return;
   int serverRetries = currentServerList->getTimeoutNum ( serverAddress );
   if ( serverRetries > retries || serverRetries < 0 )
   {
      currentServerList->setTimeout ( serverAddress );
      pingProgress->setProgress ( pingProgress->progress() + 1 );
      numSockets--;
      if ( pingProgress->progress() == pingProgress->totalSteps())
      {
         setStopped ( TRUE );
         statusLabel->setText ( "Done" );
	 pingProgress->reset();
      }
      else if ( numSockets < maxSockets && address.begin() != address.end())
      {
         refreshNextServer();
	 numSockets++;
      }
   }
   else
   {
      address.push_front ( serverAddress );
      refreshNextServer();
   }
}

void BrowseWindow::finishedQuery ( QString serverAddress )
{
   if ( pingProgress->progress() == pingProgress->totalSteps()
      && pingProgress->totalSteps() != 0 )
   {
      setStopped ( TRUE );
      statusLabel->setText ( "Done" );
      pingProgress->reset();
   }
   delete queryList[serverAddress];
   queryList.remove ( serverAddress );
   updatePlayerServerCount();
   numSockets--;
   if ( numSockets < maxSockets && address.begin() != address.end())
   {
      refreshNextServer();
      numSockets++;
   }
}

//-------------------------------------------------------------------
//  Receives ping time when pinging single server. Updates serverList
//-------------------------------------------------------------------
void BrowseWindow::updatePing ( int time )
{
   currentServerList->updateCurrentPing ( time );
}

//-------------------------------------------------------------------
//  Queries the currently selected server using gamespy
//-------------------------------------------------------------------
void BrowseWindow::refreshSingle ()
{
   // Begin refresh and make sure stopped is false
   statusLabel->setText ( "Refreshing..." );
   currentServerList = ( ServerListView* )servers->visibleWidget();
   QString serverAddress = currentServerList->getCurrentAddress();
   if ( serverAddress == NULL )
      return;
   if ( useNewGspy )
      queryList.insert ( serverAddress, new NewGSpyQuery());
   else
      queryList.insert ( serverAddress, new GSpyQuery());
   queryList[serverAddress]->setAddress ( serverAddress );
   queryList[serverAddress]->setTimeout ( timeout );
   queryList[serverAddress]->setServerList ( currentServerList );
   queryList[serverAddress]->start();
   queryList[serverAddress]->sendQuery ( serverAddress );
   connect ( queryList[serverAddress], SIGNAL ( finished ( QString )),
             SLOT ( finishedSingleQuery ( QString )));
   connect ( queryList[serverAddress], SIGNAL ( time ( int, QString )),
             SLOT ( updatePing ( int )));
   connect ( queryList[serverAddress], SIGNAL ( timeoutError ( QString )),
             SLOT ( refreshSingleTimeout ( QString )));
}

void BrowseWindow::refreshSingleTimeout ( QString serverAddress )
{
   delete queryList[serverAddress];
   queryList.remove ( serverAddress );
   currentServerList->setTimeout ( serverAddress );
   statusLabel->setText ( "Timeout" );
}

void BrowseWindow::finishedSingleQuery ( QString serverAddress )
{
   delete queryList[serverAddress];
   queryList.remove ( serverAddress );
   currentServerList->clearPlayerSettingLists();

   updatePlayerServerCount ( );
   currentServerList->updatePlayersSettings();
   statusLabel->setText ( "Done" );
   // Signal for refreshed server while attempting to connect
   emit refreshed();
}

//-------------------------------------------------------------------
//  Apply client side filters to a new list on startup or when a new
//  list is downloaded
//-------------------------------------------------------------------
void BrowseWindow::filterList ()
{
   QStringList match;
   QString key;
   QListViewItemIterator it ( andromeda );
   // Reset all servers to visible
   serversAndro->setServerVisible ( TRUE );
   it = it.current()->firstChild();
   while ( it.current()->parent() == andromeda &&
      it.current() != buddies )
   {
      key = it.current()->text(0);
      it = it.current()->firstChild();
      while ( it.current()->parent() != andromeda && 
         it.current() != buddies )
      {
         QCheckListItem *item = dynamic_cast<QCheckListItem*>(it.current());

         if ( item && !item->isOn())
            match.append ( item->text(0));
         it++;
      }
      if ( match.begin() != match.end() )
         serversAndro->setServerVisible ( key, match, FALSE );
      match.clear();
   }
   updatePlayerServerCount ();
}

//-------------------------------------------------------------------
//  Draw the currently selected serverlist or buddylist's player and
//  server counts on the statusbar
//-------------------------------------------------------------------
void BrowseWindow::updatePlayerServerCount ()
{
   ServerListView *currList = ( ServerListView* )servers->visibleWidget();
   statsLabel->setText ( QString::number ( currList->getServerCount()) + 
      " Servers / " + QString::number ( currList->getPlayerCount()) + " Players");
}

void BrowseWindow::updateBuddyCount ()
{
   int total = buddyList->numberOfBuddies();
   int online = buddyList->numberOfOnlineBuddies();
   statsLabel->setText ( QString::number ( total ) + " Buddies / " +
      QString::number ( online ) + " Online" );
}

//-------------------------------------------------------------------
//  Check if server is full, then load up dialog or connect if not full
//-------------------------------------------------------------------
void BrowseWindow::attemptConnect()
{
   ServerListView *currList = ( ServerListView* )servers->visibleWidget();
   if ( !currList->isServerSelected())
      return;
      
   if ( currList->getCurrentPlayers() < currList->getCurrentMaxPlayers())
      connectToGameServer ( currList->getCurrentAddress());
   else
   {
      retry = new RetryDialog ( this, "retry dialog", FALSE,
         Qt::WDestructiveClose );
      QString name = currList->getCurrentName();
      if ( name.startsWith ( "<qt>" ))
         name = currList->getCurrentAddress();
      retry->setCaption ("Connecting to " + name + "...");
      retry->show();
      connect ( this, SIGNAL ( refreshed()), this, SLOT ( connectRetry()));
      connect ( retry, SIGNAL ( tryAgain()), SLOT ( refreshSingle()));
      connect ( retry, SIGNAL ( cancel()), SLOT ( cancelConnect()));
      connect ( retry, SIGNAL ( connectServer()), SLOT ( forceConnect()));
      retry->startTimer();
   }
}

void BrowseWindow::connectToGameServer ( QString arg )
{
   cancelConnect ();
   QProcess *proc = new QProcess ( this );
   proc->addArgument ( armyExec );
   if ( passwordEdit->isModified())
      arg = arg + "?password=" + passwordEdit->text();
   proc->addArgument ( arg );
   if ( !proc->start())
      statusLabel->setText ( "Could not start Americas Army" );
}

//-------------------------------------------------------------------
//  Signal activated when user presses on cancel button
//-------------------------------------------------------------------
void BrowseWindow::cancelConnect ()
{
   // Stop the refresh signal from triggering a connect
   disconnect ( this, SIGNAL ( refreshed()), 0, 0 );
}

//-------------------------------------------------------------------
//  Connect to server even if full
//-------------------------------------------------------------------
void BrowseWindow::forceConnect ()
{
   ServerListView *currList = ( ServerListView* )servers->visibleWidget();
   connectToGameServer ( currList->getCurrentAddress());
}

void BrowseWindow::connectRetry()
{
   ServerListView *currList = ( ServerListView* )servers->visibleWidget();
   if ( currList->getCurrentPlayers() < currList->getCurrentMaxPlayers())
   {
      retry->connecting();
      connectToGameServer( currList->getCurrentAddress());
   }
   else
      retry->resetProgress();
}

//-------------------------------------------------------------------
//  Add listviewitems and icons to selection menu
//-------------------------------------------------------------------
void BrowseWindow::setupSelectList ()
{
   QPixmap favIcon;
   favIcon = QPixmap ( favourites_xpm );
   favourites = new QListViewItem ( selectList, "Favourites" );
   favourites->setPixmap ( 0, favIcon );
   andromeda = new QListViewItem ( selectList, favourites, "Master Server" );

   QCheckListItem *previousItem;
   QCheckListItem *currentItem;
   // Location filters
   QStringList locationList;
   locationList << "West Coast" << "East Coast" << "Euro" << "Other";
   QCheckListItem *localController = new QCheckListItem ( andromeda, "Location",
      QCheckListItem::CheckBoxController );
   previousItem = localController;
   for ( QStringList::Iterator it = locationList.begin();
      it != locationList.end(); ++it )
   {
      currentItem = new QCheckListItem ( localController, previousItem,
         *it, QCheckListItem::CheckBox );
      currentItem->setOn ( TRUE );
      previousItem = currentItem;
   }

   // Map filters
   QStringList mapList;
   mapList << "Bridge Crossing" << "Bridge SE" << "Collapsed Tunnel" <<
      "FLS Assault" << "Headquarters Raid" << "Insurgent Camp" <<
      "JRTC Farm Raid" << "Mout McKenna" << "Mountain Ambush" <<
      "Mountain Pass" << "Mountain Pass SE" << "Pipeline" << "Pipeline SF" <<
      "Radio Tower" << "River Basin" << "SF Arctic" << "SF CSAR" << 
      "SF Hospital" << "SF Oasis" << "SF Recon" << "SF SandStorm" << 
      "SF Taiga" << "SF Village" << "Swamp Raid" << "Weapons Cache" << 
      "Weapons Cache SE";
   QCheckListItem *mapController = new QCheckListItem ( andromeda,
      localController, "Map", QCheckListItem::CheckBoxController );
   previousItem = mapController;
   for ( QStringList::Iterator it = mapList.begin(); it != mapList.end(); ++it )
   {
      currentItem = new QCheckListItem ( mapController, previousItem,
         *it, QCheckListItem::CheckBox );
      currentItem->setOn ( TRUE );
      previousItem = currentItem;
   }
   
   // Version filters
   QStringList versionList;
   versionList << "2.0.0" << "2.1.0" << "2.2.0" << "2.2.1";
   QCheckListItem *versionController = new QCheckListItem ( andromeda,
      mapController, "Version", QCheckListItem::CheckBoxController );
   previousItem = versionController;
   for ( QStringList::Iterator it = versionList.begin();
      it != versionList.end(); ++it )
   {
      currentItem = new QCheckListItem ( versionController, previousItem,
         *it, QCheckListItem::CheckBox );
      currentItem->setOn ( TRUE );
      previousItem = currentItem;
   }

   // Server type filters
   QStringList typeList;
   typeList << "Official" << "Leased Official" << "Leased" << "Authorized";
   QCheckListItem *typeController = new QCheckListItem ( andromeda,
      versionController, "Server Type", QCheckListItem::CheckBoxController );
   previousItem = typeController;
   for ( QStringList::Iterator it = typeList.begin(); it != typeList.end();
      ++it )
   {
      currentItem = new QCheckListItem ( typeController, previousItem,
         *it, QCheckListItem::CheckBox );
      currentItem->setOn ( TRUE );
      previousItem = currentItem;
   }

   // Ping filters
   QStringList pingList;
   pingList << "1 - 99" << "100 - 199" << "200 - 299" << "300 - 499" <<
      "500+" << "Timeout";
   QCheckListItem *pingController = new QCheckListItem ( andromeda,
      typeController, "Ping", QCheckListItem::CheckBoxController );
   previousItem = pingController;
   for ( QStringList::Iterator it = pingList.begin(); it != pingList.end();
      ++it )
   {
      currentItem = new QCheckListItem ( pingController, previousItem,
         *it, QCheckListItem::CheckBox );
      currentItem->setOn ( TRUE );
      previousItem = currentItem;
   }

#ifdef USE_GEOIP
   // Country filters
   QStringList countryList;
   countryList << "Africa" << "Asia" << "Europe" << "North America" <<
   "South America" << "Oceania";
   QCheckListItem *countryFilter = new QCheckListItem ( andromeda,
      typeController, "Country", QCheckListItem::CheckBoxController );
   previousItem = countryFilter;
   for ( QStringList::Iterator it = countryList.begin();
      it != countryList.end(); ++ it )
   {
      currentItem = new QCheckListItem ( countryFilter, previousItem,
         *it, QCheckListItem::CheckBox );
      currentItem->setOn ( TRUE );
      previousItem = currentItem;
   }
#endif	 
	  
   andromeda->setOpen ( TRUE );

   QPixmap buddyIcon;
   buddyIcon = QPixmap ( buddies_xpm );
   buddies = new QListViewItem ( selectList, andromeda, "Buddy List" );
   buddies->setPixmap ( 0, buddyIcon );
   
}

void BrowseWindow::selectListRMB ( QListViewItem *, const QPoint& point,
   int )
{
   QPopupMenu *selectMenu = new QPopupMenu ( this );
   
   selectMenu->insertItem ( "Load filters", loadFilter );
   selectMenu->insertItem ( "Save filters", saveFilter );
   selectMenu->insertItem ( "Rename filters", renameFilter );
   selectMenu->popup ( point );
}

//-------------------------------------------------------------------
//  Save each client side filter whose value is false.
//-------------------------------------------------------------------
void BrowseWindow::saveFilters ( int id )
{
   QSettings settings;
   settings.insertSearchPath ( QSettings::Unix, QDir::homeDirPath() +
      SETTINGS_DIR );
      
   QString set;
   if ( id == s1 )
   {
      loadFilter->setItemVisible ( l1, TRUE );
      set = "Set 1";
   }
   else if ( id == s2 )
   {
      loadFilter->setItemVisible ( l2, TRUE );
      set = "Set 2";
   }
   else if ( id == s3 )
   {
      loadFilter->setItemVisible ( l2, TRUE );
      set = "Set 3";
   }
   else if ( id == s4 )
   {
      loadFilter->setItemVisible ( l2, TRUE );
      set = "Set 4";
   }

   QListViewItemIterator it ( andromeda );
   it = it.current()->firstChild();
   while ( it.current()->parent() == andromeda &&
      it.current() != buddies )
   {
      it = it.current()->firstChild();
      while ( it.current()->parent() != andromeda &&
         it.current() != buddies )
      {
         QCheckListItem *item = dynamic_cast<QCheckListItem*>(it.current());
         if ( !item->isOn() )
            settings.writeEntry ( APP_KEY + set + "/" + item->text(0), false );
	 else
	    settings.removeEntry ( APP_KEY + set + "/" + item->text(0));
         it++;
      }
   }
}

//-------------------------------------------------------------------
//  Load the saved client side filters. Sets all filters true by default
//  and false only if they have been saved
//-------------------------------------------------------------------
void BrowseWindow::loadFilters ( int id )
{
   QSettings settings;
   settings.insertSearchPath ( QSettings::Unix, QDir::homeDirPath() +
                               SETTINGS_DIR );
   QString set;
   if ( id == l1 )
      set = "Set 1";
   else if ( id == l2 )
      set = "Set 2";
   else if ( id == l3 )
      set = "Set 3";
   else if ( id == l4 )
      set = "Set 4";
   else if ( id == l0 )
      set = "";

   QStringList keys = settings.entryList ( APP_KEY + set );
   QListViewItemIterator it ( andromeda );
   it = it.current()->firstChild();
   while ( it.current()->parent() == andromeda &&
      it.current() != buddies )
   {
      it = it.current()->firstChild();
      while ( it.current()->parent() != andromeda &&
         it.current() != buddies )
      {
         QCheckListItem *item = dynamic_cast<QCheckListItem*>(it.current());
	 item->setOn ( TRUE );
         for ( QStringList::Iterator list = keys.begin(); list != keys.end();
            list++ )
         {
            if ( *list == it.current()->text(0))
               item->setOn ( FALSE );
         }
         it++;
      }
   }
   filterList();
}

void BrowseWindow::renameFilters ( int id )
{
   QSettings *settings = new QSettings();
   settings->insertSearchPath ( QSettings::Unix, QDir::homeDirPath() +
                               SETTINGS_DIR );
   QString set;
   if ( id == r1 )
      set = "Set 1";
   else if ( id == r2 )
      set = "Set 2";
   else if ( id == r3 )
      set = "Set 3";
   else if ( id == r4 )
      set = "Set 4";

   bool ok;
   QString name = QInputDialog::getText ( "Rename filter", set,
      QLineEdit::Normal, QString::null, &ok, this );
   if ( ok && !name.isEmpty())
      settings->writeEntry ( APP_KEY + set + "/" + "name", name );   
   delete settings;
   reloadSettings();
}

//-------------------------------------------------------------------
//  Called whenever the program exits. Saves window geometry,
//  favorites and serverdata for both lists.
//-------------------------------------------------------------------
void BrowseWindow::saveSettings ()
{
   QSettings settings;
   settings.insertSearchPath ( QSettings::Unix, QDir::homeDirPath() +
      SETTINGS_DIR );

   if ( saveLists )
      serversAndro->saveServerData ( "master.xml" );
   serversFavs->saveServerData ( "favorites.xml" );
   // Save current buddies to a file 
   buddyList->writeBuddies();

   // Save the possition of the server filter buttons
   QListViewItemIterator it ( andromeda );
   it = it.current()->firstChild();
   while ( it.current()->parent() == andromeda &&
      it.current() != buddies )
   {
      it = it.current()->firstChild();
      while ( it.current()->parent() != andromeda &&
         it.current() != buddies )
      {
         QCheckListItem *item = dynamic_cast<QCheckListItem*>(it.current());
         if ( item )
	    settings.writeEntry ( APP_KEY + "Modifiers/" + item->text(0),
	       item->isOn());
	 it++;
      }
   }
   // Save window geometry
   settings.writeEntry ( APP_KEY + "Settings/" + "WindowWidth", width());
   settings.writeEntry ( APP_KEY + "Settings/" + "WindowHeight", height());
   settings.writeEntry ( APP_KEY + "Settings/" + "WindowX", x());
   settings.writeEntry ( APP_KEY + "Settings/" + "WindowY", y());

   // Save toolbar positions
   QString str;
   QTextStream s ( &str, IO_ReadWrite );
   s << *this;
   settings.writeEntry ( APP_KEY + "Settings/" + "Toolbars", str );
}

//-------------------------------------------------------------------
//  Load settings, favorites, and andromeda data on startup
//-------------------------------------------------------------------
void BrowseWindow::loadStartupSettings ( )
{
   QSettings settings;
   settings.insertSearchPath ( QSettings::Unix, QDir::homeDirPath() +
      SETTINGS_DIR );

   // Load server filter settings
   QStringList keys = settings.entryList ( APP_KEY + "Modifiers/" );
   QListViewItemIterator it ( andromeda );
   it = it.current()->firstChild();
   while ( it.current()->parent() == andromeda &&
      it.current() != buddies )
   {
      it = it.current()->firstChild();
      while ( it.current()->parent() != andromeda &&
         it.current() != buddies )
      {
         for ( QStringList::Iterator list = keys.begin(); list != keys.end();
	    list++ )
	 {
	    if ( *list == it.current()->text(0))
	    {
	       QCheckListItem *item =
	          dynamic_cast<QCheckListItem*>(it.current());
	       item->setOn ( settings.readBoolEntry ( APP_KEY + "Modifiers/" +
	          *list, TRUE ));
	    }
	 }
	 it++;
      }
   }

   // Load window geometry
   int windowWidth = settings.readNumEntry ( APP_KEY + "Settings/" + 
      "WindowWidth", 640 );
   int windowHeight = settings.readNumEntry ( APP_KEY + "Settings/" +
      "WindowHeight", 480 );
   int windowX = settings.readNumEntry ( APP_KEY + "Settings/" + "WindowX" );
   int windowY = settings.readNumEntry ( APP_KEY + "Settings/" + "WindowY" );

   // Load toolbar positions
   QString str = settings.readEntry ( APP_KEY + "Settings/" + "Toolbars" );
   QTextStream s ( &str, IO_ReadWrite );
   s >> *this;
   

   resize ( windowWidth, windowHeight );
   if ( windowX != -1 || windowY != -1 )
      move ( windowX, windowY );

   serversFavs->readServerData ( "favorites.xml" );
   serversAndro->readServerData ( "master.xml" );
   updatePlayerServerCount ( );

   filterList();
   reloadSettings();
   if ( refreshOnStartup )
      refreshList();
}

//-------------------------------------------------------------------
//  Reload appliaction settings, at startup and whenever settings have changed
//-------------------------------------------------------------------
void BrowseWindow::reloadSettings()
{
   QSettings settings;
   settings.insertSearchPath ( QSettings::Unix, QDir::homeDirPath() +
      SETTINGS_DIR );

   armyExec = settings.readEntry ( APP_KEY + "Settings/" + "army executable" );
   refreshOnStartup = settings.readBoolEntry ( APP_KEY + "Settings/" + 
      "refresh on startup", FALSE );
   saveLists = settings.readBoolEntry  ( APP_KEY + "Settings/" +
      "save lists on exit", TRUE );
   maxSockets = settings.readNumEntry ( APP_KEY + "Settings/" + "sockets", 20 );
   timeout = settings.readNumEntry ( APP_KEY + "Settings/" + 
      "refresh timeout", 3 );
   retries = settings.readNumEntry ( APP_KEY + "Settings/" + "retries", 0 );
   useNewGspy = settings.readBoolEntry ( APP_KEY + "Settings/" + 
      "new gamespy protocol", TRUE );
   bool useColouredNames = settings.readBoolEntry ( APP_KEY + "Settings/" +
      "coloured names", TRUE );
   updateTimeout = settings.readNumEntry ( APP_KEY + "Settings/" + 
      "master timeout", 10 );
   masterType = settings.readNumEntry ( APP_KEY + "Settings/" +
      "master type", 1 );
   gspyMaster = settings.readEntry ( APP_KEY + "Settings/" + "gamespy master",
      "master.gamespy.com" );
   androMaster = settings.readEntry ( APP_KEY + "Settings/" + "andro master",
      "master0.supercomputerinc.com" );
   aamslMaster = settings.readEntry ( APP_KEY + "Settings/" + "aamsl master",
      "http://www.aamsl.net/index.php?" );
   refreshMaster = settings.readBoolEntry ( APP_KEY + "Settings/" +
      "master refresh", FALSE );
   QString set1 = settings.readEntry ( APP_KEY + "Set 1/name", "Set 1" );
   QString set2 = settings.readEntry ( APP_KEY + "Set 2/name", "Set 2" );
   QString set3 = settings.readEntry ( APP_KEY + "Set 3/name", "Set 3" );
   QString set4 = settings.readEntry ( APP_KEY + "Set 4/name", "Set 4" );
   saveFilter->changeItem ( s1, set1 );
   saveFilter->changeItem ( s2, set2 );
   saveFilter->changeItem ( s3, set3 );
   saveFilter->changeItem ( s4, set4 );
   loadFilter->setItemVisible ( l0, TRUE );
   bool is1, is2, is3, is4;
   is1 = !settings.entryList ( APP_KEY + "Set 1" ).empty();
   if ( is1 )
   {
      loadFilter->changeItem ( l1, set1 );
      renameFilter->changeItem ( r1, set1 );
   }
   loadFilter->setItemVisible ( l1, is1 );
   renameFilter->setItemVisible ( r1, is1 );
   is2 = !settings.entryList ( APP_KEY + "Set 2" ).empty();
   if ( is2 )
   {
      loadFilter->changeItem ( l2, set2 );
      renameFilter->changeItem ( r2, set2 );
   }
   loadFilter->setItemVisible ( l2, is2 );
   renameFilter->setItemVisible ( r2, is2 );
   is3 = !settings.entryList ( APP_KEY + "Set 3" ).empty();
   {
      loadFilter->changeItem ( l3, set3 );
      renameFilter->changeItem ( r3, set3 );
   }
   loadFilter->setItemVisible ( l3, is3 );
   renameFilter->setItemVisible ( r3, is3 );
   is4 = !settings.entryList ( APP_KEY + "Set 4" ).empty();
   if ( is4 )
   {
      loadFilter->changeItem ( l4, set4 );
      renameFilter->changeItem ( r4, set4 );
   }
   loadFilter->setItemVisible ( l4, is4 );
   renameFilter->setItemVisible ( r4, is4 );
   QPixmap masterPixmap;
   if ( masterType == 1 )
   {
      gspyAction->setOn ( TRUE );
      masterPixmap = QPixmap ( gamespy_xpm );
   }
   else if ( masterType == 2 )
   {
      androAction->setOn ( TRUE );
      masterPixmap = QPixmap ( andromeda_xpm );
   }
   else if ( masterType == 3 )
   {
      aamslAction->setOn ( TRUE );
      masterPixmap = QPixmap ( aamsl_xpm );
   }
   andromeda->setPixmap ( 0, masterPixmap );

   bool text;
   int toolbarType = settings.readNumEntry ( APP_KEY + "Settings/" +
      "toolbutton type", 1 );
   if ( toolbarType == 1 )
      text = TRUE;
   else
      text = FALSE;
   setUsesTextLabel ( text );

   serversAndro->setColouredNames ( useColouredNames );
   serversFavs->setColouredNames ( useColouredNames );
}

//-------------------------------------------------------------------
//  Create right click menu when clicking on serverlist.
//-------------------------------------------------------------------
void BrowseWindow::serverRMB ( QListViewItem *item, const QPoint & point, int )
{
   QPopupMenu *serverMenu = new QPopupMenu ( this );
   refreshSelAction->setEnabled ( item );
   refreshCurAction->setEnabled ( item );
   delCurAction->setEnabled ( item );
   copyAction->setEnabled ( item );
   connectAction->setEnabled ( item );
   if ( item )
   {

      refreshCurAction->addTo ( serverMenu );
      refreshSelAction->addTo ( serverMenu );

      serverMenu->insertSeparator ();

      connectAction->addTo ( serverMenu );
      copyAction->addTo ( serverMenu );

      serverMenu->insertSeparator ();

      delCurAction->addTo ( serverMenu );
   }
   delTimeAction->addTo ( serverMenu );

   if ( selectList->currentItem()->text ( 0 ) == "Favourites" )
   {
      serverMenu->insertSeparator();
      addServerAction->setEnabled ( TRUE );
      addFavsAction->setEnabled ( FALSE );
      addServerAction->addTo ( serverMenu );
   }
   else
   {
      delAllAction->addTo ( serverMenu );
      serverMenu->insertSeparator();
      addServerAction->setEnabled ( FALSE );
      addFavsAction->setEnabled ( item );
      if ( item )
	 addFavsAction->addTo ( serverMenu );
   }
   serverMenu->popup ( point );
}

void BrowseWindow::serverLMB ( QListViewItem *item )
{
   refreshSelAction->setEnabled ( item );
   refreshCurAction->setEnabled ( item );
   delCurAction->setEnabled ( item );
   copyAction->setEnabled ( item );
   connectAction->setEnabled ( item );
   if ( selectList->currentItem()->text(0) != "Favourites" )
      addFavsAction->setEnabled ( item );
}

//-------------------------------------------------------------------
//  Create right click menu when clicking on playerlist.
//-------------------------------------------------------------------
void BrowseWindow::playerRMB ( QListViewItem *item, const QPoint & point, int )
{
   if ( item )
   {
      QPopupMenu *playerMenu = new QPopupMenu ( this );
      if ( !item->pixmap( 0 ))
         playerMenu->insertItem ( "Add player to buddy list", this, 
            SLOT ( addToBuddyList ()));
      else
	 playerMenu->insertItem ( "Delete player from buddy list", this,
            SLOT ( deleteFromBuddyList ()));
      playerMenu->popup ( point );
   }
}

//-------------------------------------------------------------------
//  Add the selected player to the buddylist
//-------------------------------------------------------------------
void BrowseWindow::addToBuddyList()
{
   ServerListView *currList = ( ServerListView* )servers->visibleWidget();
   buddyList->saveBuddy ( currList->getCurrentPlayerName(), 
      currList->getCurrentAddress(), currList->getCurrentName());
   currList->setPlayerBuddyIcon ( TRUE );
}

//-------------------------------------------------------------------
//  Remove the selected player from the buddylist
//-------------------------------------------------------------------
void BrowseWindow::deleteFromBuddyList()
{
   ServerListView *currList = ( ServerListView* )servers->visibleWidget();
   buddyList->deleteBuddy ( currList->getCurrentPlayerName());
   currList->setPlayerBuddyIcon ( FALSE );
}

//-------------------------------------------------------------------
//  Copy the selected server from andromeda to the favorites list
//-------------------------------------------------------------------
void BrowseWindow::moveToFavs ()
{
   if ( addFavsAction->isEnabled())
   {
      QString address = serversAndro->getCurrentAddress ();
      serversFavs->addNewServer ( address );
      serversFavs->selectCurrentServer ();
   }
}

//-------------------------------------------------------------------
//  Brings up a dialog for adding a new server to favorites
//-------------------------------------------------------------------
void BrowseWindow::getNewServer ()
{
   AddServerDialog *newAddress = new AddServerDialog ( this, "add server",
      FALSE, Qt::WDestructiveClose );
   newAddress->setCaption ( "Add new server..." );
   newAddress->show();
   
   connect ( newAddress, SIGNAL ( address ( QString )), 
      SLOT ( addNewServer ( QString )));
}

void BrowseWindow::addNewServer( QString address )
{
   if ( addServerAction->isEnabled())
   {
      serversFavs->addNewServer ( address );
      updatePlayerServerCount ( );
   }
}

void BrowseWindow::deleteServer ()
{
   if ( delCurAction->isEnabled())
   {
      ServerListView *currList = ( ServerListView* )servers->visibleWidget();
      currList->deleteCurrentServer();
      updatePlayerServerCount ( );
   }
}

//-------------------------------------------------------------------
//  Copy server address from serverlist to X selection buffer.
//-------------------------------------------------------------------
void BrowseWindow::copyServerAddr ()
{
   if ( copyAction->isEnabled())
   {
      ServerListView *currList = ( ServerListView* )servers->visibleWidget();
      QClipboard *cb = QApplication::clipboard();
      cb->setText ( currList->getCurrentAddress(), QClipboard::Selection );
   }
}

void BrowseWindow::removeTimedoutServers()
{
   if ( delTimeAction->isEnabled())
   {
      ServerListView *currList = ( ServerListView* )servers->visibleWidget();
      currList->removeTimedoutServers();
      updatePlayerServerCount();
   }
}

void BrowseWindow::clearServerList()
{
   if ( delAllAction->isEnabled())
   {
      ServerListView *currList = ( ServerListView* )servers->visibleWidget();
      currList->clearServerList();
      updatePlayerServerCount();
   }
}
