// browsewindow.h
//
/****************************************************************************
   Copyright (C) 2004, David Henry. All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*****************************************************************************/

#ifndef BROWSEWINDOW_H
#define BROWSEWINDOW_H

#include <qmainwindow.h>
#include <qdict.h>

class ServerListView;
class QAction;
class QListView;
class QListViewItem;
class HttpQuery;
class Gsms;
class GSpyMaster;
class RetryDialog;
class QProgressBar;
class QPopupMenu;
class QLineEdit;
class QWidgetStack;
class QLabel;
class BuddyList;
class Query;

class BrowseWindow: public QMainWindow
{
   Q_OBJECT

public:
   BrowseWindow();
   ~BrowseWindow();

signals:
   void refreshed();

private slots:
   void about();
   void pref ();
   void stop ();
   void filterSettings();
   void playerSearch ();
   void changeServerView ( QListViewItem *newServerList );
   void findServer ( QString, QString );
   void changeMaster ( QAction * );

   // Andromeda and http queries
   void updateAndromedaList();
   void parseServerList ( QString );
   void parseHttpQuery ( bool );
   void numServers ( int );
   void updateBytes ( int );
   void finishedMaster ();
   void masterTimeout();

   // Gamespy queries
   void refreshList ();
   void refreshSelected ();
   void refreshTimeout ( QString );
   void refreshPing ( int, QString );
   void updatePing ( int );
   void refreshSingle ();
   void refreshSingleTimeout ( QString );
   void finishedQuery ( QString );
   void finishedSingleQuery ( QString );

   // Joining servers
   void attemptConnect();
   void connectToGameServer( QString );
   void cancelConnect();
   void forceConnect();
   void connectRetry();

   void saveSettings ();
   void reloadSettings();
   void selectListRMB ( QListViewItem *, const QPoint &, int );
   void serverRMB ( QListViewItem *, const QPoint &, int );
   void playerRMB ( QListViewItem *, const QPoint &, int );
   void serverLMB ( QListViewItem * );
   void saveFilters ( int );
   void loadFilters ( int );
   void renameFilters ( int );
   void getNewServer ();
   void addNewServer ( QString);
   void deleteServer ();
   void copyServerAddr ( );
   void addToBuddyList();
   void deleteFromBuddyList();
   void moveToFavs();
   void clearServerList();
   void removeTimedoutServers();

private:
   // Toolbars
   QAction *refreshAllAction, *refreshSelAction, *refreshCurAction;
   QAction *delAllAction, *delTimeAction, *delCurAction;
   QAction *updateAction, *filterAction, *settingsAction;
   QAction *copyAction, *connectAction, *addServerAction;
   QAction *stopAction, *addFavsAction, *searchAction;
   QAction *gspyAction, *androAction, *aamslAction;
   QPopupMenu *loadFilter, *saveFilter, *renameFilter;

   void checkActions();

   // Dialogs
   RetryDialog *retry;

   // Listviews
   QWidgetStack *servers;
   ServerListView *serversFavs;
   ServerListView *serversAndro;
   BuddyList *buddyList;
   QListView *selectList;
   QListViewItem *favourites;
   QListViewItem *andromeda;
   QListViewItem *buddies;
   void setupSelectList();
   void filterList ();

   // Server queries
   void refreshNextServer();
   QStringList address;
   int numSockets;
   GSpyMaster *masterGspy;
   Gsms *masterGsms;
   HttpQuery *masterQuery;
   ServerListView *currentServerList;
   QDict<Query> queryList;
   bool updating;

   // Status bar
   QProgressBar *pingProgress;
   QLabel *statusLabel;
   QLabel *statsLabel;
   
   void loadStartupSettings ();

   QLineEdit *passwordEdit;

   void updatePlayerServerCount ( );
   void updateBuddyCount ();

   bool stopped;
   void setStopped ( bool );
   bool isStopped ();

   // Settings
   QString armyExec;
   bool refreshOnStartup;
   bool saveLists;
   int maxSockets;
   int timeout;
   int retries;
   bool useNewGspy;
   int updateTimeout;
   int masterType;
   QString gspyMaster;
   QString androMaster;
   QString aamslMaster;
   bool refreshMaster;

   // Menu item id's
   int l0, l1, l2, l3, l4;
   int s1, s2, s3, s4;
   int r1, r2, r3, r4;
};

#endif
