// buddylist.cpp
//
/****************************************************************************
   Copyright (C) 2004, David Henry. All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*****************************************************************************/

#include "buddylist.h"

#include <qlistview.h>
#include <qdir.h>
#include <qpopupmenu.h>
#include <qpixmap.h>
#include <qdom.h>
#include <qdatetime.h>
#include <qapplication.h>
#include <qdragobject.h>
#include <qsimplerichtext.h>

#include "../xpms/button_ok.xpm"

const QString SETTINGS_DIR = "/.aabrowse/";
const QString APP_KEY = "/aabrowse/";

BuddyList::BuddyList ( QWidget *parent=0 )
   : QListView ( parent ), mousePressed ( FALSE )
{
   addColumn ( "Player Name" );
   addColumn ( "Online" );
   addColumn ( "Current Address" );
   setColumnWidthMode ( 2, QListView::Manual );
   hideColumn ( 2 );
   addColumn ( "Server Name", 300 );
   addColumn ( "Last Seen" );

   setRootIsDecorated ( TRUE );
   setAcceptDrops ( TRUE );
   viewport()->setAcceptDrops ( TRUE );

   individuals =  new QListViewItem ( this );
   individuals->setText ( 0, "Individuals" );
   individuals->setText ( 1, "0 / 0" );
   individuals->setOpen ( TRUE );

   list = new QDict<BuddyListItem> ( 307, FALSE );

   loadBuddies ();

   connect ( this, SIGNAL ( contextMenuRequested ( QListViewItem *,
      const QPoint&, int )), this, SLOT ( buddyRMB ( QListViewItem *,
      const QPoint&, int )));
   connect ( this, SIGNAL ( itemRenamed ( QListViewItem *, int,
      const QString & )), this, SLOT ( buddyRenamed ( QListViewItem *, int,
      const QString & )));
}

BuddyList::~BuddyList()
{
   delete list;
}

void BuddyList::buddyRMB ( QListViewItem *item , const QPoint &point, int )
{
   buddyMenu = new QPopupMenu ( this );
   if ( item  && item->parent())
   {
      if ( !item->text( 2 ).isEmpty())
      {
         buddyMenu->insertItem ( "Show server", this, SLOT ( showServer()));
         buddyMenu->insertItem ( "Join this server", this, SLOT (joinServer()));
         buddyMenu->insertSeparator ();
      }
      buddyMenu->insertItem ( "Delete player", this,
         SLOT ( deleteCurrentBuddy()), Key_Delete);
   }
   if ( item && !item->parent() && item->childCount() == 0 )
      buddyMenu->insertItem ( "Delete group", this, SLOT ( deleteGroup()));

      
   buddyMenu->insertItem ( "Add new player", this, SLOT ( addBuddy()));
   buddyMenu->insertItem ( "Add new group", this, SLOT ( addGroup()));
   buddyMenu->popup ( point );
   
}

//-------------------------------------------------------------------
//  On statup parse the buddy XML file and add values to listview
//-------------------------------------------------------------------
void BuddyList::loadBuddies()
{
   QFile buddyFile ( QDir::homeDirPath() + SETTINGS_DIR + "buddies.xml" );
   if ( buddyFile.open ( IO_ReadOnly ))
   {
      QDomDocument doc ( "buddy list" );
      doc.setContent ( &buddyFile );
      QDomNode node = doc.firstChild();
      node = node.firstChild();
      while ( !node.isNull())
      {
	 if ( node.isElement() && node.nodeName() == "Buddy")
	 {
	    BuddyListItem *newBuddy = new BuddyListItem ( individuals );
	    newBuddy->setDragEnabled ( TRUE );

	    QDomNode buddyNode = node.firstChild();
	    while ( !buddyNode.isNull() && buddyNode.isElement())
	    {
	       if ( buddyNode.nodeName() == "Group" )
	       {
	          QString groupName = buddyNode.toElement().text();
		  if ( !groupName.isEmpty())
		  {
		     //  Add the new group if it doesnt exist
		     QListViewItem *groupItem = findGroup ( groupName );
		     if ( !groupItem )
		     {
		        groupItem = new QListViewItem ( this );
		        groupItem->setText ( 0, groupName );
			groupItem->setText ( 1, "0 / 0" );
		     }
                     //  Move buddie to its group
		     individuals->takeItem ( newBuddy );
		     groupItem->insertItem ( newBuddy );
		  }
		  increaseBuddies ( newBuddy->parent(), TRUE );
	       }
	       if ( buddyNode.nodeName() == "Name" )
	       {
	          QString name = buddyNode.toElement().text();
		  newBuddy->setText ( 0, name );
		  list->insert ( name, newBuddy );
	       }
	       if ( buddyNode.nodeName() == "Address" )
	          newBuddy->setText ( 2, buddyNode.toElement().text());
	       if ( buddyNode.nodeName() == "Server" )
	          newBuddy->setText ( 3, buddyNode.toElement().text());
	       if ( buddyNode.nodeName() == "Time" )
	          newBuddy->setText ( 4, buddyNode.toElement().text());
	       buddyNode = buddyNode.nextSibling();
	    }
	    QDomElement e = node.toElement();
	 }
         node = node.nextSibling();
      }
      buddyFile.close();
   }
}

//-------------------------------------------------------------------
//  Adds buddy name, address and serverName to the current listview
//-------------------------------------------------------------------
void BuddyList::saveBuddy ( QString name, QString address, QString serverName )
{
   QDateTime time;
   time = time.currentDateTime();
   
   BuddyListItem *newBuddy = new BuddyListItem ( individuals );
   list->insert ( name, newBuddy );
   newBuddy->setDragEnabled ( TRUE );
   newBuddy->setText ( 0, name );
   newBuddy->setText ( 2, address );
   newBuddy->setText ( 3, serverName );
   newBuddy->setText ( 4, time.toString ( "dd.MM.yy hh:mm" ));
   QPixmap onlineIcon;
   onlineIcon = QPixmap ( button_ok_xpm );
   newBuddy->setPixmap ( 1, onlineIcon );
   increaseOnlineBuddies ( individuals, TRUE );
   increaseBuddies ( individuals, TRUE );
}

//-------------------------------------------------------------------
//  On exit creates an XML file containing all buddy values
//-------------------------------------------------------------------
void BuddyList::writeBuddies ()
{
   QFile buddyFile ( QDir::homeDirPath() + SETTINGS_DIR + "buddies.xml" );
   QTextStream stream ( &buddyFile );
   QDomDocument doc ( "Buddies" );
   QDomElement root = doc.createElement ( "Buddies" );
   doc.appendChild ( root );
   QListViewItemIterator it ( this );
   while ( it.current())
   {
      if ( it.current()->parent())
      {
         QDomElement buddyElement = doc.createElement ( "Buddy" );
         root.appendChild ( buddyElement );
      
         QDomElement groupElement = doc.createElement ( "Group" );
         buddyElement.appendChild ( groupElement );
         QDomText groupText = doc.createTextNode ( 
            it.current()->parent()->text(0));
	 groupElement.appendChild ( groupText );
	 
         QDomElement nameElement = doc.createElement ( "Name" );
         buddyElement.appendChild ( nameElement );
         QDomText nameText = doc.createTextNode ( it.current()->text(0));
         nameElement.appendChild ( nameText );

         QDomElement serverElement = doc.createElement ( "Server" );
         buddyElement.appendChild ( serverElement );
         QDomText serverText = doc.createTextNode ( it.current()->text(3));
         serverElement.appendChild ( serverText );

         QDomElement addressElement = doc.createElement ( "Address" );
         buddyElement.appendChild ( addressElement );
         QDomText addressText = doc.createTextNode ( it.current()->text(2));
         addressElement.appendChild ( addressText );

         QDomElement timeElement = doc.createElement ( "Time" );
         buddyElement.appendChild ( timeElement );
         QDomText timeText = doc.createTextNode ( it.current()->text (4));
         timeElement.appendChild ( timeText );
      }
      it++;
   }
   if ( buddyFile.open ( IO_WriteOnly ))
   {
      stream << doc.toString();
      buddyFile.close();
   }
}

//-------------------------------------------------------------------
//  Returns true if buddy is found in the current listview
//-------------------------------------------------------------------
bool BuddyList::isBuddy ( QString buddy )
{
   if ( (*list)[buddy] != 0 )
      return TRUE;
   return FALSE;
}

QListViewItem *BuddyList::findGroup ( QString group )
{
   QListViewItemIterator it ( this );
   while ( it.current ())
   {
      if ( !it.current()->parent())
      {
         if ( it.current()->text(0) == group )
	    return it.current();
      }
      it++;
   }
   return NULL;
}
//-------------------------------------------------------------------
//  Finds buddy in the listview and updates values
//-------------------------------------------------------------------
void BuddyList::updateBuddy ( QString name, QString address, QString serverName )
{
   QListViewItemIterator it ( this );
   while ( it.current())
   {
      if ( it.current()->text(0).lower() == name.lower() )
      {
	 if ( !it.current()->pixmap( 1 ))
	    increaseOnlineBuddies ( it.current()->parent(), TRUE );
	 QDateTime time;
	 time = time.currentDateTime();
	 QPixmap onlineIcon;
	 onlineIcon = QPixmap ( button_ok_xpm );
	 it.current()->setPixmap ( 1, onlineIcon );
         it.current()->setText ( 2, address );
	 it.current()->setText ( 3, serverName );
	 it.current()->setText ( 4, time.toString ( "dd.MM.yy hh:mm" ));
	 return;
      }
      it++;
   }
}

void BuddyList::deleteBuddy ( QString name )
{
   QListViewItemIterator it ( this );
   while ( it.current() && it.current()->text ( 0 ).lower() != name.lower() )
      it++;

   if ( it.current()->pixmap ( 1 ))
      increaseOnlineBuddies ( it.current()->parent(), FALSE );
   increaseBuddies ( it.current()->parent(), FALSE );
   
   it.current()->~QListViewItem();
   list->remove ( name );
}

void BuddyList::deleteCurrentBuddy ()
{
   if ( !this->currentItem()->parent())
      return;
   if ( currentItem()->pixmap ( 1 ))
      increaseOnlineBuddies ( currentItem()->parent(), FALSE );
   increaseBuddies ( currentItem()->parent(), FALSE );

   list->remove ( this->currentItem()->text(0));
   this->currentItem()->~QListViewItem();
}

void BuddyList::addGroup ()
{
   QListViewItem *newClan = new QListViewItem ( this );
   newClan->setRenameEnabled ( 0, TRUE );
   newClan->startRename ( 0 );
   newClan->setText ( 1, "0 / 0" );
}

void BuddyList::addBuddy ()
{
   QListViewItem *newBuddy;
   if ( this->selectedItem())
   {
      if ( this->selectedItem()->parent())
         newBuddy = new BuddyListItem ( this->selectedItem()->parent());
      else
         newBuddy = new BuddyListItem ( this->selectedItem());
   }
   else
      newBuddy = new BuddyListItem ( individuals );
   newBuddy->setDragEnabled ( TRUE );
   newBuddy->setRenameEnabled( 0, TRUE );
   newBuddy->startRename ( 0 );
}

void BuddyList::buddyRenamed ( QListViewItem *item, int, const QString &name )
{
   BuddyListItem *buddyItem = dynamic_cast<BuddyListItem*>(item);
   if ( buddyItem )
      list->insert ( name, buddyItem );
}

void BuddyList::deleteGroup ()
{
   if ( !this->currentItem()->parent())
      this->currentItem()->~QListViewItem();
}

void BuddyList::joinServer ()
{
   emit join ( this->currentItem()->text(2));
}

void BuddyList::showServer ()
{
   emit raiseServer ( this->currentItem()->text(2),
      this->currentItem()->text(0));
}

int BuddyList::numberOfBuddies()
{
   int num = 0;
   QListViewItemIterator it ( this );
   while ( it.current())
   {
      if ( it.current()->parent())
         num++;
      it++;
   }
   return num;
}

int BuddyList::numberOfOnlineBuddies()
{
   int num = 0;
   QListViewItemIterator it ( this );
   while ( it.current())
   {
      if ( it.current()->pixmap ( 1 ))
         num++;
      it++;
   }
   return num;
}

void BuddyList::increaseOnlineBuddies ( QListViewItem *groupName, bool positive)
{
   QString buddyNumbers = groupName->text ( 1 );
   int numBuddies = buddyNumbers.section ( " / ", 0, 0 ).toInt();

   if ( positive )
      numBuddies++;
   else
      numBuddies--;

   groupName->setText ( 1, QString::number ( numBuddies ) + " / " +
      buddyNumbers.section ( " / ", 1, 1 ));
}

void BuddyList::increaseBuddies ( QListViewItem *groupName, bool positive )
{
   QString buddyNumbers = groupName->text ( 1 );
   int numBuddies = buddyNumbers.section ( " / ", 1, 1 ).toInt();

   if ( positive )
      numBuddies++;
   else
      numBuddies--;

   groupName->setText ( 1, buddyNumbers.section ( " / ", 0, 0 ) + " / " +
      QString::number ( numBuddies ));
}

void BuddyList::contentsMousePressEvent ( QMouseEvent *event )
{
   QListView::contentsMousePressEvent ( event );
   QPoint p ( contentsToViewport ( event->pos()));
   QListViewItem *item = itemAt ( p );
   if ( item && event->button() == Qt::LeftButton )
   {
      pressPos = event->pos();
      draggedItem = item;
      mousePressed = TRUE;
   }
}

void BuddyList::contentsMouseMoveEvent ( QMouseEvent *event )
{
   if ( mousePressed && ( pressPos - event->pos()).manhattanLength() >
      QApplication::startDragDistance())
   {
      mousePressed = FALSE;
      QListViewItem *item = itemAt ( contentsToViewport ( pressPos ));
      if ( item && item->parent() )
         startDrag();
   }
}

void BuddyList::contentsMouseReleaseEvent ( QMouseEvent * )
{
   mousePressed = FALSE;
}

void BuddyList::startDrag ()
{
   clearSelection();
   QTextDrag *d = new QTextDrag ( this );
   d->drag();
}

void BuddyList::contentsDropEvent ( QDropEvent *event )
{
   QListViewItem *newItem = itemAt ( contentsToViewport ( event->pos()));

   if ( newItem->parent() )
      newItem = newItem->parent();

   increaseBuddies ( newItem, TRUE );
   increaseBuddies ( draggedItem->parent(), FALSE );
   if ( draggedItem->pixmap ( 1 ))
   {
      increaseOnlineBuddies ( newItem, TRUE );
      increaseOnlineBuddies ( draggedItem->parent(), FALSE );
   }

   draggedItem->parent()->takeItem ( draggedItem );
   newItem->insertItem ( draggedItem );
}

BuddyListItem::BuddyListItem ( QListViewItem *prevItem )
	: QListViewItem ( prevItem )
{
}

void BuddyListItem::paintCell ( QPainter *p, const QColorGroup &cg,
   int column, int width, int align )
{
   if ( column == 3 )
   {
      QString name = text ( column );
      if ( name.startsWith ( "<qt>" ))
      {
         QBrush br = cg.brush ( QColorGroup::Base );

         QSimpleRichText tmp ( name, listView()->font());
	 tmp.draw ( p, 0, -2, QRect ( 0, 0, width, height()), cg, &br );
      }
      else
         QListViewItem::paintCell ( p, cg, column, width, align );
   }
   else
      QListViewItem::paintCell ( p, cg, column, width, align );
}
