// buddylist.h
//
/****************************************************************************
   Copyright (C) 2004, David Henry. All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*****************************************************************************/

#ifndef BUDDYLIST_H
#define BUDDYLIST_H

#include <qlistview.h>
#include <qdict.h>

class QPopupMenu;
class QPainter;
class QColorGroup;

class BuddyListItem : public QListViewItem
{
public:
   BuddyListItem ( QListViewItem * );
   void paintCell ( QPainter *, const QColorGroup &, int, int, int );
};

class BuddyList : public QListView
{
   Q_OBJECT

public:
   BuddyList( QWidget *parent );
   ~BuddyList();
   void saveBuddy ( QString, QString, QString );
   void updateBuddy ( QString, QString, QString );
   void deleteBuddy ( QString name );
   void writeBuddies ();
   bool isBuddy ( QString buddy );
   int numberOfBuddies ();
   int numberOfOnlineBuddies();

signals:
   void join ( QString );
   void raiseServer ( QString, QString );

protected:
   virtual void contentsMousePressEvent ( QMouseEvent * );
   virtual void contentsMouseMoveEvent ( QMouseEvent * );
   virtual void contentsMouseReleaseEvent ( QMouseEvent * );
   virtual void startDrag ();
   virtual void contentsDropEvent ( QDropEvent * );

private slots:
   void buddyRMB ( QListViewItem *, const QPoint &, int );
   void addBuddy ();
   void joinServer ();
   void showServer ();
   void deleteCurrentBuddy();
   void addGroup ();
   void deleteGroup ();
   void buddyRenamed ( QListViewItem *, int, const QString & );

private:
   QListViewItem *individuals;
   QPopupMenu *buddyMenu;
   void loadBuddies ();
   QListViewItem *findGroup ( QString );
   void increaseOnlineBuddies ( QListViewItem *, bool );
   void increaseBuddies ( QListViewItem *, bool );

   bool mousePressed;
   QPoint pressPos;
   QListViewItem *draggedItem;
   QDict<BuddyListItem> *list;
};

#endif
