// filterdialog.cpp
//
/****************************************************************************
   Copyright (C) 2004, David Henry. All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*****************************************************************************/

#include "filterdialog.h"

#include <qlayout.h>
#include <qvbuttongroup.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qspinbox.h>
#include <qsettings.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qgrid.h>
#include <qdir.h>

const QString SETTINGS_DIR = "/.aabrowse/";
const QString APP_KEY = "/aabrowse/";

FilterDialog::FilterDialog ( QWidget *parent=0, const char *name=0,
   bool modal = FALSE, WFlags f=0 )
   : QDialog ( parent, name, modal, f )
{
   QSettings settings;
   settings.insertSearchPath ( QSettings::Unix, QDir::homeDirPath() + 
      SETTINGS_DIR );

   masterType = settings.readNumEntry ( APP_KEY + "Settings/" +
      "master type", 1 );
   mainBox = new QVBoxLayout ( this );
   mainBox->setMargin ( 5 );

   // Type and map comboboxes
   pickGrid = new QGrid ( 2, this );
   pickGrid->setSpacing ( 5 );
   
   mapCheckBox = new QCheckBox ( "Map:", pickGrid );
   typeCheckBox = new QCheckBox ( "Type:", pickGrid );

   mapPick = new QComboBox ( FALSE, pickGrid );
   mapPick->insertItem ( "Bridge Crossing" );
   mapPick->insertItem ( "Bridge SE" );
   mapPick->insertItem ( "Collapsed Tunnel" );
   mapPick->insertItem ( "FLS Assault" );
   mapPick->insertItem ( "Headquarters Raid" );
   mapPick->insertItem ( "Insurgent Camp" );
   mapPick->insertItem ( "JRTC Farm" );
   mapPick->insertItem ( "MOUT McKenna" );
   mapPick->insertItem ( "Mountain Ambush" );
   mapPick->insertItem ( "Mountain Pass" );
   mapPick->insertItem ( "Mountain Pass SE" );
   mapPick->insertItem ( "Pipeline" );
   mapPick->insertItem ( "Pipeline SF" );
   mapPick->insertItem ( "Radio Tower" );
   mapPick->insertItem ( "River Basin" );
   mapPick->insertItem ( "SF Arctic" );
   mapPick->insertItem ( "SF CSAR" );
   mapPick->insertItem ( "SF Hospital" );
   if ( masterType != 3 )
      mapPick->insertItem ( "SF Oasis" );
   mapPick->insertItem ( "SF Recon" );
   mapPick->insertItem ( "SF Sandstorm" );
   if ( masterType != 3 )
      mapPick->insertItem ( "SF Taiga" );
   mapPick->insertItem ( "SF Village" );
   mapPick->insertItem ( "Swamp Raid" );
   mapPick->insertItem ( "Weapons Cache" );
   mapPick->insertItem ( "Weapons Cache SE" );

   typePick = new QComboBox ( FALSE, pickGrid );
   typePick->insertItem ( "All Honor" );
   typePick->insertItem ( "Official" );
   typePick->insertItem ( "Leased Official" );
   typePick->insertItem ( "Leased" );
   typePick->insertItem ( "Authorized" );

   mainBox->addWidget ( pickGrid );
   
   buttonBox = new QHBoxLayout ( mainBox );
   checkButtonGroup = new QVButtonGroup ( this );
   checkButtonGroup->setFrameStyle ( 0 );

   buttonBox->addWidget ( checkButtonGroup );

   // Checkboxes - use different boxes if using http
   if ( masterType == 1 || masterType == 3 )
   {
      fullCheckBox = new QCheckBox ( "Show full", checkButtonGroup );
      emptyCheckBox = new QCheckBox ( "Show empty", checkButtonGroup );
   }
   if ( masterType == 1 || masterType == 2 )
   {
      milesCheckBox = new QCheckBox ( "Show miles", checkButtonGroup );
      cheatCheckBox = new QCheckBox ( "Show cheats", checkButtonGroup );
   }
   privateCheckBox = new QCheckBox ( "Show private", checkButtonGroup );

   rightBox = new QVBoxLayout ( buttonBox );
   rightBox->setSpacing ( 5 );

   QGrid *spinBox = new QGrid ( 2, this );
   spinBox->setSpacing ( 5 );
   rightBox->addWidget ( spinBox );

   // Don't add honor checking for http
   if ( masterType != 3 )
   {
      minHonorLabel = new QLabel ( "Minimum Honor:", spinBox );
      minHonorSpinBox = new QSpinBox ( 1, 100, 1, spinBox );
      minHonorSpinBox->setValue ( 1 );

      maxHonorLabel = new QLabel ( "Maximum Honor:", spinBox );
      maxHonorSpinBox = new QSpinBox ( 1, 100, 1, spinBox );
      maxHonorSpinBox->setValue ( 100 );
   }
   minPlayersLabel = new QLabel ( "Minimum Players:", spinBox );
   minPlayersSpinBox = new QSpinBox ( 0, 26, 1, spinBox );
   minPlayersSpinBox->setValue ( 0 );

   maxPlayersLabel = new QLabel ( "Maximum Players:", spinBox );
   maxPlayersSpinBox = new QSpinBox ( 1, 26, 1, spinBox );
   maxPlayersSpinBox->setValue ( 26 );

   mainBox->addSpacing ( 10 );
   
   // OK and cancel buttons
   controlBox = new QHBoxLayout ( mainBox );
   controlBox->addSpacing ( 200 );
   okButton = new QPushButton ( "OK", this );
   cancelButton = new QPushButton ( "Cancel", this );

   controlBox->addWidget ( okButton );
   controlBox->addWidget ( cancelButton );

   bool isEmpty = settings.readBoolEntry ( APP_KEY + "Filters/" + "empty",
         TRUE );
   bool isFull = settings.readBoolEntry ( APP_KEY + "Filters/" + "full",
         TRUE );
   bool isCheat = settings.readBoolEntry ( APP_KEY + "Filters/" + "cheats", 
         FALSE );
   bool isMiles = settings.readBoolEntry ( APP_KEY + "Filters/" + "miles",
         TRUE );
   int minHonor = settings.readNumEntry ( APP_KEY + "Filters/" + 
         "minhonor", 1 );
   int maxHonor = settings.readNumEntry ( APP_KEY + "Filters/" + 
         "maxhonor", 100 );
   bool isPrivate = settings.readBoolEntry ( APP_KEY + "Filters/" +
      "MBS_PRIVATE", TRUE );
   int minPlayers = settings.readNumEntry ( APP_KEY + "Filters/" +
      "MBS_PLAYERS", 0 );
   int maxPlayers = settings.readNumEntry ( APP_KEY + "Filters/" +
      "MBS_MAXPLAYERS", 26 );
   QString map = settings.readEntry ( APP_KEY + "Filters/" + "map" );
   QString type = settings.readEntry ( APP_KEY + "Filters/" + "type" );
   
   if ( masterType != 3 )
   {
      cheatCheckBox->setChecked ( isCheat );
      milesCheckBox->setChecked ( isMiles );

      minHonorSpinBox->setValue ( minHonor );
      maxHonorSpinBox->setValue ( maxHonor );
   }
   if ( masterType == 1 || masterType == 3 )
   {
      fullCheckBox->setChecked ( isFull );
      emptyCheckBox->setChecked ( isEmpty );
   }
   privateCheckBox->setChecked ( isPrivate );
   minPlayersSpinBox->setValue ( minPlayers );
   maxPlayersSpinBox->setValue ( maxPlayers );

   int i = 0;
   //  Find map in the list and display it as current item
   if ( !map.isEmpty())
   {
      mapCheckBox->setChecked ( TRUE );
      while ( i < mapPick->count() )
      {
         if ( mapPick->text ( i ) == map )
            mapPick->setCurrentItem ( i );
         i++; 
      }
   }
   i = 0;
   if ( !type.isEmpty())
   {
      typeCheckBox->setChecked ( TRUE );
      while ( i < typePick->count() )
      {
         if ( typePick->text ( i ) == type )
            typePick->setCurrentItem ( i );
         i++;
      }
   }
   connect ( okButton, SIGNAL (pressed()), this, SLOT ( accept()));
   connect ( cancelButton, SIGNAL (pressed()), SLOT ( reject()));
}

//-------------------------------------------------------------------
//  Save current settings to file when OK is clicked
//-------------------------------------------------------------------
void FilterDialog::accept()
{
   QSettings settings;
   settings.insertSearchPath ( QSettings::Unix, QDir::homeDirPath() + 
      SETTINGS_DIR );
   if ( masterType == 1 || masterType == 3 )
   {
      settings.writeEntry ( APP_KEY + "Filters/" + "full",
         fullCheckBox->isChecked());
      settings.writeEntry ( APP_KEY + "Filters/" + "empty",
         emptyCheckBox->isChecked());
   }
   if ( masterType == 1 || masterType == 2 )
   {
      settings.writeEntry ( APP_KEY + "Filters/" + "cheats", 
         cheatCheckBox->isChecked());
      settings.writeEntry ( APP_KEY + "Filters/" + "miles",
         milesCheckBox->isChecked());
      settings.writeEntry ( APP_KEY + "Filters/" + "minhonor", 
         minHonorSpinBox->value());
      settings.writeEntry ( APP_KEY + "Filters/" + "maxhonor", 
         maxHonorSpinBox->value());
   }
   settings.writeEntry ( APP_KEY + "Filters/" + "MBS_PRIVATE",
      privateCheckBox->isChecked());
   settings.writeEntry ( APP_KEY + "Filters/" + "MBS_PLAYERS",
      minPlayersSpinBox->value());
   settings.writeEntry ( APP_KEY + "Filters/" + "MBS_MAXPLAYERS",
      maxPlayersSpinBox->value());

   QString map, type;
   if ( mapCheckBox->isChecked ())
      map = mapPick->currentText();
   else 
      map = "";
   settings.writeEntry ( APP_KEY + "Filters/" + "map", map );
   
   if ( typeCheckBox->isChecked ())
      type = typePick->currentText();
   else
      type = "";
   settings.writeEntry ( APP_KEY + "Filters/" + "type", type );

   QDialog::accept();
}

