// gsmsXMLdoc.cpp
//
/****************************************************************************
   Copyright (C) 2004, David Henry. All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*****************************************************************************/

#include "gsmsXMLdoc.h"
#include "serverlistview.h"

GsmsXMLDoc::GsmsXMLDoc( const QString &name )
   : QDomDocument ( name )
{
   leased = FALSE;
   official = FALSE;
}

void GsmsXMLDoc::parseServerList ( ServerListView *currList )
{
   int i = 0;
   QDomNodeList addrList = elementsByTagName ( "MBS_IPADDRESS" );
   QDomNodeList portList = elementsByTagName ( "MBS_HOSTPORT" );
   QDomElement root = documentElement();
   QDomNode node = root.firstChild();
   node = node.firstChild();

   // Skip first elements
   do {
      node = node.nextSibling();
   } while ( !node.isNull() && node.nodeName() != "RESPONSE" );

   // Skip status messages
   node = node.firstChild();
   while ( !node.isNull() && node.nodeName() != "SERVERS" )
      node = node.nextSibling();

   node = node.firstChild();
   while ( !node.isNull())
   {
      // Found a server
      if ( node.isElement() && node.nodeName() == "SERVER" )
      {
         QDomNode serverNode = node.firstChild();
	 QDomNode addr = addrList.item ( i );
	 QDomNode port = portList.item ( i );
	 if ( currList->addNewServer ( addr.toElement().text() + ":" + 
	      port.toElement().text()))
         {
	    // Server does not exist so it must be added
	    official = FALSE;
	    leased = FALSE;
	    while ( !serverNode.isNull() && serverNode.isElement())
	    {
               if ( serverNode.nodeName() == "MBS_HOSTNAME" )
	          currList->setServerName ( serverNode.toElement().text());
               else if ( serverNode.nodeName() == "MBS_PLAYERS" )
	          currList->setServerNumPlayers (serverNode.toElement().text());
               else  if ( serverNode.nodeName() == "MBS_MAXPLAYERS" )
	          currList->setServerMaxPlayers (serverNode.toElement().text());
               else if ( serverNode.nodeName() == "MBS_MAP" )
	          currList->setServerMap ( serverNode.toElement().text());
               else if ( serverNode.nodeName() == "SV_PUNKBUSTER" )
               {
                  if ( serverNode.toElement().text() == "1" )
	             currList->setServerPB ( TRUE );
	          else
	             currList->setServerPB ( FALSE );
	       }
               else if ( serverNode.nodeName() == "TOUR" )
	          currList->setServerTour ( serverNode.toElement().text());
               else if ( serverNode.nodeName() == "LEASED" )
               {
                  if ( serverNode.toElement().text() == "1" )
                  {
		     leased = TRUE;
		     if ( official == TRUE )
		        currList->setServerHonor ( TRUE );
		     else
	   	        currList->setServerLeased ( TRUE );
	          }
	          else if ( official == FALSE )
	             currList->setServerLeased ( FALSE );
	       }
               else if ( serverNode.nodeName() == "OFFICIAL" )
               {
                  if ( serverNode.toElement().text() == "1" )
                  {
		     official = TRUE;
		     if ( leased == TRUE )
		        currList->setServerHonor ( TRUE );
		     else
		        currList->setServerOfficial ( TRUE );
	          }
	          else if ( leased == FALSE )
	             currList->setServerOfficial ( FALSE );
	       }
               else if ( serverNode.nodeName() == "MILES" )
               {
                  if ( serverNode.toElement().text() == "1" )
	             currList->setServerMiles ( TRUE );
               }
               else if ( serverNode.nodeName() == "MBS_PRIVATE" )
               {
                  if ( serverNode.toElement().text() == "1" )
	             currList->setServerPrivate ( TRUE );
               }
	       serverNode = serverNode.nextSibling();
	    }
	 }
      }
      i++;
      node = node.nextSibling();
   }
}
