// gspymaster.cpp
//
/****************************************************************************
   Copyright (C) 2004, David Henry. All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
     
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*****************************************************************************/

#include "gspymaster.h"
#include "gsmsalg.h"
#include "serverlistview.h"

#include <qtimer.h>
#include <qsettings.h>
#include <qdir.h>

const QString key = "g3sR2b";
const QString SETTINGS_DIR = "/.aabrowse/";
const QString APP_KEY = "/aabrowse/";

GSpyMaster::GSpyMaster( int serverTimeout )
	: QSocket ()
{
   timer = new QTimer ( this );
   connect ( timer, SIGNAL ( timeout()), this, SLOT ( timeout()));
   timer->start ( serverTimeout * 1000, TRUE );
   connect ( this, SIGNAL ( readyRead()), this, SLOT ( readData()));
   connect ( this, SIGNAL ( connectionClosed()), this, SLOT ( finished()));
}

void GSpyMaster::setServerList ( ServerListView *list )
{
   currList = list;
}

void GSpyMaster::readData()
{
   timer->stop();
   int length = 0;
   length = bytesAvailable();
   int oldLength = data.size();
   data.resize ( length + oldLength + 1);
   readBlock ( data.data() + oldLength, length );
   data[length + oldLength] = '\0';
   
   if ( data.contains ( "\\basic\\\\secure\\" ))
   {
      data[length-1] = '\0';
      QString secure = data.data();
      QString decryptedKey;
      secure = secure.section ( "\\", 4, 4 );
      data.resize ( 0 );

      decryptedKey = (const char *)gsseckey ( secure, key, 1 );
      sendKey ( decryptedKey );
   }
   else
   {
      int newLength = length + oldLength;
      if ( data[newLength-1] == '\\' && data[newLength-2] == 'l' &&
         data[newLength-3] == 'a' && data[newLength-4] == 'n' &&
	 data[newLength-5] == 'i' && data[newLength-6] == 'f' )
      {
	 newLength -= 7;
      }
      else
      {
         QDataStream stream ( data, IO_ReadOnly );
	 Q_UINT32 a;
	 Q_UINT16 p;
	 int newLength = length + oldLength;
         while ( newLength >= 6 )
	 {
	    stream >> a;
	    stream >> p;
	    newLength -= 6;
	    QHostAddress addr;
	    addr.setAddress ( a );
	    QString port = QString::number ( p-1 );
	    currList->addNewServer ( addr.toString() + ":" + port );
	 }
	 for ( int i = length - newLength; i < length; i++ )
	    data[i-length+newLength] = data[i];
	 data.resize ( newLength );
      }
   }
}

void GSpyMaster::sendKey ( QString unlockedKey )
{
   QString finalString = "\\basic\\gamename\\armygame\\enctype\\0\\validate\\"
      + unlockedKey + "\\final\\\\queryid\\1.1\\list\\cmp\\gamename\\armygame"
      + "\\where\\" + buildFilterList() + "\\final\\";

   writeBlock ( finalString, finalString.length());
}

void GSpyMaster::timeout()
{
   close();
   emit serverTimeout();
}

void GSpyMaster::finished()
{
   emit queryFinished();
}

QString GSpyMaster::buildFilterList()
{
   QSettings settings;
   settings.insertSearchPath ( QSettings::Unix, QDir::homeDirPath() +
      SETTINGS_DIR );
   int minHonor = settings.readNumEntry ( APP_KEY + "Filters/" +
      "minhonor", 1 );
   int maxHonor = settings.readNumEntry ( APP_KEY + "Filters/" +
      "maxhonor", 100 );
   int minPlayers = settings.readNumEntry ( APP_KEY + "Filters/" +
      "MBS_PLAYERS", 0 );
   int maxPlayers = settings.readNumEntry ( APP_KEY + "Filters/" +
      "MBS_MAXPLAYERS", 26 );
   QString map = settings.readEntry ( APP_KEY + "Filters/" + "map" );
   QString type = settings.readEntry ( APP_KEY + "Filters/" + "type" );
   bool miles = settings.readBoolEntry ( APP_KEY + "Filters/" + "miles", 1 );
   bool cheat = settings.readBoolEntry ( APP_KEY + "Filters/" + "cheats", 0 );
   bool locked = settings.readBoolEntry ( APP_KEY + "Filters/" +
      "MBS_PRIVATE", 0 );
   bool isEmpty = settings.readBoolEntry ( APP_KEY + "Filters/" + "epmty",
      TRUE );
   bool isFull = settings.readBoolEntry ( APP_KEY + "Filters/" + "full",
      TRUE );

   QString filterList;
   bool first = TRUE;
   if ( minHonor > 1 )
   {
      filterList += "(minhonor >= " + QString::number ( minHonor ) + ")";
      first = FALSE;
   }
   if ( maxHonor < 100 )
   {
      if ( !first )
         filterList += " AND ";
      first = FALSE;
      filterList += "(maxhonor <= " + QString::number ( maxHonor ) + ")";
   }
   if ( minPlayers > 0 )
   {
      if ( !first )
         filterList += " AND ";
      first = FALSE;
      filterList += "(numplayers > " + QString::number ( minPlayers ) + ")";
   }
   else if ( !isEmpty )
   {
      if ( !first )
         filterList += " AND ";
      first = FALSE;
      filterList += "(numplayers > 0)";
   }
   if ( !isFull )
   {
      if ( !first )
         filterList += " AND ";
      first = FALSE;
      filterList += "(numplayers != maxplayers)";
   }
   if ( maxPlayers < 26 )
   {
      if ( !first )
         filterList += " AND ";
      first = FALSE;
      filterList += "(numplayers < " + QString::number ( maxPlayers ) + ")";
   }
   if ( !map.isEmpty())
   {
      if ( !first )
         filterList += " AND ";
      first = FALSE;
      filterList += "(mapname = '" + map + "')";
   }
   if ( !type.isEmpty())
   {
      if ( !first )
         filterList += " AND ";
      first = FALSE;
      if ( type == "All Honor" )
         filterList += "(official = 1)";
      else
      {
         QString oValue, lValue;
         if ( type == "Authorized" || type == "Official" )
	    lValue = "0";
	 else
	    lValue = "1";
	 if ( type == "Leased" || type == "Authorized" )
	    oValue = "0";
	 else
	    oValue = "1";
	 filterList += "(official = " + oValue + ") AND (leased = " + lValue +
	    ")";
      }
   }
   if ( !miles )
   {
      if ( !first )
         filterList += " AND ";
      first = FALSE;
      filterList += "(miles = 0)";
   }
   if ( !cheat )
   {
      if ( !first )
         filterList += " AND ";
      first = FALSE;
      filterList += "(cheats = 0)";
   }
   if ( !locked )
   {
      if ( !first )
         filterList += " AND ";
      first = FALSE;
      filterList += "(password = 0)";
   }
   return filterList;
}
