// httpquery.cpp
//
/****************************************************************************
   Copyright (C) 2004, David Henry. All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*****************************************************************************/

#include "httpquery.h"

#include <qurloperator.h>
#include <qnetwork.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qdir.h>
#include <qsettings.h>

const QString SETTINGS_DIR = "/.aabrowse/";
const QString APP_KEY = "/aabrowse/";

HttpQuery::HttpQuery()
{
   bytes = 0;
   // Remove any old file if it exists
   QFile dataFile ( QDir::homeDirPath() + SETTINGS_DIR + "aamsl.php" );
   dataFile.remove ();

   // Connect to aamsl website and download serverlist
   qInitNetworkProtocols();
   op = new QUrlOperator ( "http://www.aamsl.net/index.php?"
      + getFilters());
   op->get();

   connect ( op, SIGNAL ( data ( const QByteArray &, QNetworkOperation * )),
      this, SLOT ( httpData ( const QByteArray &, QNetworkOperation * )));
   connect ( op, SIGNAL ( finished ( QNetworkOperation * )),
      this, SLOT ( httpFinished ( QNetworkOperation * )));
}

HttpQuery::~HttpQuery()
{
   delete op;
}

//-------------------------------------------------------------------
//  Data received is written straight to file
//-------------------------------------------------------------------
void HttpQuery::httpData( const QByteArray &data, QNetworkOperation *no )
{
   // Emit number of bytes read for each packet of data
   bytes += data.size ();
   emit bytesRead ( bytes );

   if ( !no )
      return;

   if ( no && no->state() != QNetworkProtocol::StFailed )
   {
      QFile dataFile ( QDir::homeDirPath() + SETTINGS_DIR + "aamsl.php" );

      if ( dataFile.open ( IO_WriteOnly | IO_Append ))
      {
         QTextStream stream ( &dataFile );
	 stream << QString ( data );
	 dataFile.close ();
      }
   }
} 

//-------------------------------------------------------------------
//  Slot called when data transfer has finished.
//-------------------------------------------------------------------
void HttpQuery::httpFinished ( QNetworkOperation *no )
{
   if ( no && no->state() == QNetworkProtocol::StDone )
      emit transferFinished ( TRUE );
   else
      emit transferFinished ( FALSE );
}

//-------------------------------------------------------------------
//  Constructs the query string depending on what filters are set
//-------------------------------------------------------------------
QString HttpQuery::getFilters ()
{
   QSettings settings;
   settings.insertSearchPath ( QSettings::Unix, QDir::homeDirPath() + 
      SETTINGS_DIR );
   int minPlayers = settings.readNumEntry ( APP_KEY + "Filters/" + 
      "MBS_PLAYERS", 0 );
   int maxPlayers = settings.readNumEntry ( APP_KEY + "Filters/" +
      "MBS_MAXPLAYERS", 26 );
   QString map = settings.readEntry ( APP_KEY + "Filters/" + "map" );
   QString type = settings.readEntry ( APP_KEY + "Filters/" + "type" );
   bool isEmpty = settings.readBoolEntry ( APP_KEY + "Filters/" + "empty",
      TRUE );
   bool isFull = settings.readBoolEntry ( APP_KEY + "Filters/" + "full",
      TRUE );
   bool isPrivate  = settings.readBoolEntry ( APP_KEY + "Filters/" + 
      "MBS_PRIVATE", 1 );
      
   //   Type filter
   if ( type == "" )
      type = "0";
   else if ( type == "All Honor" )
      type = "1";
   else if ( type == "Official" )
      type = "2";
   else if ( type == "Leased Official" )
      type = "3";
   else if ( type == "Leased" )
      type = "5";
   else if ( type == "Authorized" )
      type = "6";

   //   Password filter
   QString fp;
   if ( isPrivate )
      fp = "&prv=1";
   else fp = "";

   //   Empty server filter
   QString fe;
   if ( isEmpty )
      fe = "&empty=1";
   else fe = "";
   
   //   Full server filter
   QString ff;
   if ( isFull )
      ff = "&full=1";
   else ff = "";

   //   Map filter
   if ( map == "" )
      map = "0";
   else if ( map == "Bridge Crossing" )
      map = "bridge";
   else if ( map == "Bridge SE" )
      map = "bse";
   else if ( map == "Collapsed Tunnel" )
      map = "tunl";
   else if ( map == "FLS Assault" )
      map = "fls";
   else if ( map == "Headquarters Raid" )
      map = "hqr";
   else if ( map == "Insurgent Camp" )
      map = "ic";
   else if ( map == "JRTC Farm" )
      map = "jrtc";
   else if ( map == "MOUT McKenna" )
      map = "mout";
   else if ( map == "Mountain Ambush" )
      map = "ma";
   else if ( map == "Mountain Pass" )
      map = "mp";
   else if ( map == "Mountain Pass SE" )
      map = "mpse";
   else if ( map == "Pipeline" )
      map = "pipe";
   else if ( map == "Pipeline SF" )
      map = "psf";
   else if ( map == "Radio Tower" )
      map = "rt";
   else if ( map == "River Basin" )
      map = "rb";
   else if ( map == "SF Arctic" )
      map = "arct";
   else if ( map == "SF CSAR" )
      map = "csar";
   else if ( map == "SF Hospital" )
      map = "hosp";
   else if ( map == "SF Oasis" )
      map = "oasis";
   else if ( map == "SF Recon" )
      map = "recon";
   else if ( map == "SF Sandstorm" )
      map = "sfss";
   else if ( map == "SF Taiga" )
      map = "taiga";
   else if ( map == "SF Village" )
      map = "vill";
   else if ( map == "Swamp Raid" )
      map = "sr";
   else if ( map == "Weapons Cache" )
      map = "wc";
   else if ( map == "Weapons Cache SE" )
      map = "wcse";
      
   //   Max player filter
   if ( maxPlayers == 26 )
      maxPlayers = 0;
   
   QString filterString = "map=" + map + "&loc=&sort=&type=" + type + 
      "&ping=&plim=&pmin=" + QString::number ( minPlayers ) +
      "&pmax=" + QString::number ( maxPlayers ) + ff + fe + fp +
      "&fz=on";
   return filterString;
}
