// newgspyquery.cpp
//
/****************************************************************************
   Copyright (C) 2004, David Henry. All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*****************************************************************************/

#include "newgspyquery.h"
#include "serverlistview.h"

#include <qsocketdevice.h>
#include <qsocketnotifier.h>
#include <qtimer.h>
#include <qdatetime.h>
#include <qstring.h>
#include <qcstring.h>

NewGSpyQuery::NewGSpyQuery()
{
}

NewGSpyQuery::~NewGSpyQuery()
{
   delete pingTimer;
}

//-------------------------------------------------------------------
//  Send gamespy UDP status packet to server
//-------------------------------------------------------------------
void NewGSpyQuery::sendQuery ( QString address )
{
   QHostAddress addr;
   QString port;
   
   addr.setAddress ( address.section ( ':', 0,0 ));
   port = address.section ( ':', 1,1 );

   pingTimer = new QTime ();
   pingTimer->start();
   
   const char data[10] = { 0xFE, 0xFD, 0x00, 0x04, 0x05, 0x06, 0x07, 0xFF, 0xFF,
      0xFF };
   qsd->writeBlock ( data, 10, addr, port.toInt()+1);
}

//-------------------------------------------------------------------
//  Emits the final data string and emits the ping time when the first
//  packet arrives
//-------------------------------------------------------------------
void NewGSpyQuery::recvQuery (int)
{
   int ms = pingTimer->restart();
   timer->stop();
   int bytes = qsd->bytesAvailable();
   if ( bytes >= 0 )
   {
      // Need extra space for NULL terminator
      QCString recvData;
      recvData.resize ( bytes + 1 );
      bytes = qsd->readBlock ( recvData.data(), bytes );

      QString address = qsd->peerAddress().toString() + ':' + 
         QString::number ( qsd->peerPort() -1);

      emit time ( ms, address );
      parseQuery ( recvData );
   }
}

void NewGSpyQuery::parseQuery ( QCString queryString )
{
   currList->setCurrentServer ( serverAddress );
   if ( !currList->isCurrentServerSet())
      return;
   currList->setServerBuddyIcon ( FALSE );

   int length = queryString.size();
   // First key begins at position 4
   int i = 4, j = 0, k = 0;
   // Stop the loop when we get 3 00 bytes in a row
   while ( i < length && i != j-1 )
   {
      // End of first key is next 00 byte
      for ( j = i+1; j < length && queryString[j] != 0x00; j++ );

      // End of first value is next 00 byte
      for ( k = j+1; k < length && queryString[k] != 0x00; k++ );

      // Check if its a 00 byte key and 00 byte value
      if ( i != j-1 )
      {
         QString key,value;
         int pos;
         for ( pos = i+1; pos < j; pos++ )
            key[pos-i-1] = queryString[pos];
         for ( pos = j+1; pos < k; pos++ )
            value[pos-j-1] = queryString[pos];

	 if ( key == "hostname" )
	    currList->setServerName ( value );
	 else if ( key == "mapname" )
	    currList->setServerMap ( value );
	 else if ( key == "tour" )
	    currList->setServerTour ( value );
	 else if ( key == "official" )
	 {
	    if ( value == "1" )
	    {
	       official = TRUE;
	       if ( leased == TRUE )
	          currList->setServerHonor ( TRUE );
	       else
	          currList->setServerOfficial ( TRUE );
	    }
	    else if ( leased == FALSE )
	       currList->setServerOfficial ( FALSE );
	 }
	 else if ( key == "leased" )
	 {
	    if ( value == "1" )
	    {
	       leased = TRUE;
	       if ( official == TRUE )
	          currList->setServerHonor ( TRUE );
	       else
	          currList->setServerLeased ( TRUE );
	    }
	    else if ( official == FALSE )
	       currList->setServerLeased ( FALSE );
	 }
	 else if ( key == "miles" )
	 {
	    if ( value == "1" )
	       currList->setServerMiles ( TRUE );
	    else 
	       currList->setServerMiles ( FALSE );
	 }
	 else if ( key == "password" )
	 {
	    if ( value == "1" )
	       currList->setServerPrivate ( TRUE );
	    else
	       currList->setServerPrivate ( FALSE );
	 }
	 else if ( key == "sv_punkbuster" )
	 {
	    if ( value == "1" )
	       currList->setServerPB ( TRUE );
	    else
	       currList->setServerPB ( FALSE );
	 }
	 else if ( key == "maxplayers" )
	    currList->setServerMaxPlayers ( value );
	 else if ( key == "numplayers" )
	    currList->setServerNumPlayers ( value );
	 else if ( key == "gamever" )
	    currList->setSettingValue ( "Version", value );
	 else if ( key == "cheats" )
	    currList->setSettingValue ( "Cheats", value );
	 else if ( key == "minhonor" )
	    currList->setSettingValue ( "MinimumHonor", value );
	 else if ( key == "maxhonor" )
	    currList->setSettingValue ( "MaximumHonor", value );
	 else if ( key == "current_round" )
	    currList->setSettingValue ( "CurrentRound", value );
	 else if ( key == "mission_time" )
	    currList->setSettingValue ( "MissionTime", value );
	 else if ( key == "AdminEMail" )
	    currList->setSettingValue ( "AdminEmail", value );
	 else if ( key == "AdminName" )
	    currList->setSettingValue ( key, value );
	 // start again at the end of the previous key/value pair
         i = k;
      }
   }
   unsigned char numplayers = queryString[k+1];
   currList->setActualPlayerNumber ();
   j = 0;

   // Skip over the leader_ etc data
   for ( i = k+1; queryString[i] != 0x00 || queryString[i+1] != 0x00 && i < 
      length; i++ );

   if ( numplayers > 0 )
   {
      i++;
      int numFields;
      //  Loop over each player
      for ( k = 0; k < numplayers; k++ )
      {
	 // Loop over each player field
         for ( numFields = 0; numFields < 8; numFields++ )
         {
            for ( j = i+1; j < length && queryString[j] != 0x00; j++ );

            QString value;
	    int pos;
	    for ( pos = i+1; pos < j; pos++ )
	       value[pos-i-1] = queryString[pos];
           
            i = j;
            if ( numFields == 0 )
	       currList->setPlayerValue ( k, "leader", value );
	    else if ( numFields == 1 )
	       currList->setPlayerValue ( k, "goal", value );
	    else if ( numFields == 2 )
	       currList->setPlayerValue ( k, "honor", value );
	    else if ( numFields == 3 )
	       currList->setPlayerValue ( k, "player", value );
	    else if ( numFields == 4 )
	       currList->setPlayerValue ( k, "ping", value );
	    else if ( numFields == 5 )
	       currList->setPlayerValue ( k, "roe", value );
	    else if ( numFields == 6 )
	       currList->setPlayerValue ( k, "kia", value );
	    else if ( numFields == 7 )
	       currList->setPlayerValue ( k, "enemy", value );
         }
      }
   }
   // Get the team scores
   for ( ; i < length && queryString[i] == 0x00; i++ );
   unsigned char numTeams = queryString[i];
   // Skip over the score_t part
   for ( ; i < length && queryString[i] != 0x00; i++ );
   k = 0;
   for ( i=i+1; k < numTeams; k++ )
   {
      // Find each team score
      for ( j = i+1; j < length && queryString[j] != 0x00; j++ );

      QString value;
      int pos;
      for ( pos = i+1; pos < j; pos++ )
         value[pos-i-1] = queryString[pos];
      i = j;
      if ( k == 0 )
         currList->setSettingValue ( "Team1Score", value );
      else if ( k == 1 )
         currList->setSettingValue ( "Team2Score", value );
   }
   emit finished ( serverAddress );
}
