// parsehttpquery.cpp
//
/****************************************************************************
   Copyright (C) 2004, David Henry. All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*****************************************************************************/

#include "parsehttpquery.h"
#include "serverlistview.h"

#include <qfile.h>
#include <qdir.h>
#include <qtextstream.h>
#include <qstring.h>
#include <qstringlist.h>

const QString SETTINGS_DIR = "/.aabrowse/";
const QString APP_KEY = "/aabrowse/";

ParseHttpQuery::ParseHttpQuery ()
{
}

//-------------------------------------------------------------------
//  Loads the saved PHP file and finds each line that contains server data
//-------------------------------------------------------------------
void ParseHttpQuery::parseList ( ServerListView *currList, QString fileName )
{
   QFile serverList ( QDir::homeDirPath () + SETTINGS_DIR + fileName );
   if ( serverList.open ( IO_ReadOnly ))
   {
      QTextStream stream ( &serverList );
      while ( !stream.atEnd())
      {
         QString line = stream.readLine();
         if ( line.contains ( "class=\"tr_server\" " ))
	    checkLine ( currList, line );
      }
      serverList.close();
   }
   serverList.remove ( QDir::homeDirPath() + SETTINGS_DIR + fileName );
}

//-------------------------------------------------------------------
//  Parses a single line containing all server values and updates the
//  serverlist
//-------------------------------------------------------------------
void ParseHttpQuery::checkLine ( ServerListView *currList, QString line )
{
   // Beginning of a new server in the list
   QString part = line.section ( "<td>", 1, 1 );
   QString address = part.section ( "\"", 1, 1 );
   if ( address.startsWith ( "javascript" ))
      address = address.section ( "'", 1, 1 );
   else
      address = address.remove ( 0, 6 );
   // Only continue parsing if the server is not already in the list
   if ( !currList->addNewServer ( address ))
      return;

   for ( int i = 2; i <= 11; i++ )
   {
      part = line.section ( "<td", i, i );
      if ( i == 2 )
      {
         QString serverIcon = part.section ( "\"", 1, 1 );
         if ( serverIcon == "images/s/pw.gif" )
            currList->setServerPrivate ( TRUE );
         else if ( serverIcon == "images/s/l_hon.gif" )
            currList->setServerHonor ( TRUE );
         else if ( serverIcon == "images/s/off.gif" || 
               serverIcon == "images/s/nato.gif" )
            currList->setServerOfficial ( TRUE );
         else if ( serverIcon == "images/s/l_off.gif" )
            currList->setServerLeased ( TRUE );
      }
      else if ( i == 3 )
      {
         QString milesIcon = part.section ( "\"", 1, 1 );
         if ( milesIcon == "images/s/lf.gif" )
            currList->setServerMiles ( FALSE );
	 else
	    currList->setServerMiles ( TRUE );
      }
      else if ( i == 4 )
      {
         QString pbIcon = part.section ( "\"", 1, 1 );
         if ( pbIcon == "images/s/pbs.gif" )
            currList->setServerPB ( TRUE );
	 else
	    currList->setServerPB ( FALSE );
      }
      else if ( i == 7 )
      {
         QString serverName = part.section ( ">", 1, 1 );
	 serverName = serverName.left ( serverName.length()-4);
	 serverName = serverName.replace ( "&nbsp;", " ", FALSE );
         currList->setServerName ( serverName );
      }
      else if ( i == 8 )
      {
         QString mapName = part.section ( ">", 1,1 );
	 mapName = mapName.left ( mapName.length()-4 );
	 QString tour;
	 if ( mapName == "MOUT McKenna" )
	 {
	    mapName == "Mout McKenna";
	    tour = "1";
	 }
	 else if ( mapName == "Bridge" )
	 {
	    mapName = "Bridge Crossing";
	    tour = "2";
	 }
	 else if ( mapName == "Bridge SE" )
	    tour = "2";
	 else if ( mapName.startsWith ( "Mountain Pass" ))
	    tour = "2";
	 else if ( mapName == "Pipeline" )
	    tour = "2";
	 else if ( mapName == "Insurgent Camp" )
	    tour = "3";
	 else if ( mapName == "River Basin" )
	    tour = "3";
	 else if ( mapName == "HQ Raid" )
	 {
	    mapName = "Headquarters Raid";
	    tour = "3";
	 }
	 else if ( mapName == "Tunnel" )
	 {
	    mapName = "Collapsed Tunnel";
	    tour = "3";
	 }
	 else if ( mapName == "FLS" )
	 {
	    mapName = "FLS Assault";
	    tour = "6";
	 }
	 else if ( mapName == "JRTC Farm Raid" )
	    tour = "6";
	 else if ( mapName == "Swamp Raid" )
	    tour = "7";
	 else if ( mapName == "Mountain Ambush" )
	    tour = "7";
	 else if ( mapName.startsWith ( "Weapons Cache" ))
	    tour = "8";
	 else if ( mapName == "Radio Tower" )
	    tour = "8";
	 else if ( mapName == "Pipeline SF" )
	    tour = "11";
	 else if ( mapName.startsWith ( "SF " ))
	    tour = "11";
	 currList->setServerTour ( tour );
	 currList->setServerMap ( mapName );
      }
      else if ( i == 9 )
      {
         QString numPlayers = part.section ( ">", 1, 1 ).remove ( "</td" );
	 currList->setServerNumPlayers ( numPlayers );
      }
      else if ( i == 10 )
      {
         QString maxPlayers = part.section ( ">", 1, 1 ).remove ( "</td" );
	 currList->setServerMaxPlayers ( maxPlayers );
      }
   }
}
