// query.cpp
//
/****************************************************************************
   Copyright (C) 2004, David Henry. All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*****************************************************************************/
#include "query.h"
#include "serverlistview.h"

#include <qsocketdevice.h>
#include <qsocketnotifier.h>
#include <qtimer.h>

Query::Query()
{
   serverTimeout = 3;
   official = FALSE;
   leased = FALSE;
}

Query::~Query()
{
   delete qsn;
   delete qsd;
   delete timer;
}

void Query::start()
{
   qsd = new QSocketDevice ( QSocketDevice::Datagram );
   qsn = new QSocketNotifier ( qsd->socket(), QSocketNotifier::Read );

   timer = new QTimer ( this );
   connect ( timer, SIGNAL ( timeout()), this, SLOT ( timeout()));
   timer->start ( serverTimeout * 1000, TRUE );

   connect ( qsn, SIGNAL ( activated ( int )), this, SLOT ( recvQuery ( int )));
}

void Query::timeout()
{
   qsn->setEnabled ( FALSE );
   qsd->close();
   emit timeoutError( serverAddress );
}

//-------------------------------------------------------------------
//  Set the address so its possible to tell which server a timeout came from
//-------------------------------------------------------------------
void Query::setAddress ( QString address )
{
   serverAddress = address;
}

void Query::setTimeout ( int timeoutValue )
{
   serverTimeout = timeoutValue;
}

void Query::setServerList ( ServerListView *list )
{
   currList = list;
}

