// retrydialog.cpp
//
/****************************************************************************
   Copyright (C) 2004, David Henry. All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*****************************************************************************/
#include <qprogressbar.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qtimer.h>
#include <qlabel.h>
#include <qsettings.h>
#include <qdir.h>

#include "retrydialog.h"

const QString SETTINGS_DIR = "/.aabrowse/";
const QString APP_KEY = "/aabrowse/";

RetryDialog::RetryDialog ( QWidget *parent=0, const char *name=0,
   bool modal=FALSE, WFlags f=0 )
   : QDialog ( parent, name, modal, f )
{
   QSettings settings;
   settings.insertSearchPath ( QSettings::Unix, QDir::homeDirPath() +
      SETTINGS_DIR );
   // Get timeout value, default of 5
   timeoutValue = settings.readNumEntry ( APP_KEY + "Settings/" + 
      "connect timeout", 5 );
   timeoutValue *= 1000;   // Need value in milliseconds
   retryTimer = new QTimer ( this );
   mainBox = new QVBoxLayout ( this );
   mainBox->setMargin ( 5 );
   serverLabel = new QLabel ( this );
   timeProgress = new QProgressBar ( this );
   timeProgress->setTotalSteps ( timeoutValue );
   progressTimer = new QTimer ( this );

   mainBox->addWidget ( serverLabel );
   mainBox->addWidget ( timeProgress );

   mainBox->addSpacing ( 10 );

   buttonBox = new QHBoxLayout ( mainBox );
   connectButton = new QPushButton ( "Connect", this );
   cancelButton = new QPushButton ( "Cancel", this );

   buttonBox->addSpacing ( 200 );
   buttonBox->addWidget ( connectButton );
   buttonBox->addWidget ( cancelButton );

   connect ( connectButton, SIGNAL ( clicked ()), this, SLOT ( accept ()));
   connect ( cancelButton, SIGNAL ( clicked ()), this, SLOT ( reject ()));
   connect ( retryTimer, SIGNAL ( timeout ()), this, SLOT ( retry ()));
   connect ( progressTimer, SIGNAL ( timeout ()), this, SLOT ( updateProgress()));
}

void RetryDialog::retry()
{
   emit tryAgain();
}

void RetryDialog::updateProgress()
{
   timeProgress->setProgress ( timeProgress->progress() + timeoutValue/10 );
}

void RetryDialog::stopTimer ()
{
   retryTimer->stop();
   progressTimer->stop();
   timeProgress->setProgress ( timeoutValue );
}

void RetryDialog::startTimer()
{
   retryTimer->start ( timeoutValue, FALSE );
   progressTimer->start ( timeoutValue / 10, FALSE );
   timeProgress->setProgress ( 0 );
}

void RetryDialog::resetProgress()
{
   progressTimer->stop();
   progressTimer->start ( timeoutValue / 10, FALSE );
   retryTimer->stop();
   retryTimer->start ( timeoutValue, FALSE );
   timeProgress->setProgress ( 0 );
}

void RetryDialog::accept ()
{
   emit connectServer();
   stopTimer();
   QDialog::accept();
}

void RetryDialog::reject ()
{
   stopTimer();
   emit cancel();
   QDialog::reject();
}

void RetryDialog::connecting ()
{
   accept();
}
