// searchdialog.cpp
//
/****************************************************************************
   Copyright (C) 2004, David Henry. All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*****************************************************************************/

#include "searchdialog.h"
#include "serverlistview.h"
#include "buddylist.h"

#include <qlayout.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qlistview.h>
#include <qcheckbox.h>
#include <qhbox.h>
#include <qsimplerichtext.h>
#include <qpopupmenu.h>

#include "../xpms/buddy.xpm"

SearchDialog::SearchDialog ( QWidget *parent=0, const char *name=0,
   bool modal=FALSE, WFlags f=0 )
   : QDialog ( parent, name, modal, f )
{
   setCaption ( "Player search" );
   mainBox = new QVBoxLayout ( this );
   mainBox->setMargin ( 5 );
   mainBox->addSpacing ( 20 );
   
   playerEdit = new QLineEdit ( this );

   playerList = new QListView ( this );
   playerList->addColumn ( "Player" );
   playerList->addColumn ( "Server", 200 );
   playerList->addColumn ( "Address" );

   checkBox = new QHBox ( this );
   caseCheckBox = new QCheckBox ( "Case sensitive", checkBox );
   exactCheckBox = new QCheckBox ( "Match exact", checkBox );
   
   mainBox->addWidget ( playerEdit );
   mainBox->addSpacing ( 20 );
   mainBox->addWidget ( playerList );
   mainBox->addSpacing ( 20 );
   mainBox->addWidget ( checkBox );

   controlBox = new QHBoxLayout ( mainBox );
   controlBox->addSpacing ( 200 );
   searchButton = new QPushButton ( "Search", this );
   searchButton->setDefault ( TRUE );
   okButton = new QPushButton ( "OK", this );
   controlBox->addWidget ( searchButton );
   controlBox->addWidget ( okButton );

   connect ( okButton, SIGNAL ( clicked ()), this, SLOT ( accept()));
   connect ( searchButton, SIGNAL ( clicked ()), this, SLOT ( search()));
   connect ( playerList, SIGNAL ( contextMenuRequested ( QListViewItem *,
      const QPoint&, int )), this, SLOT ( playerRMB ( QListViewItem *,
      const QPoint&, int )));
}

void SearchDialog::search()
{
   playerList->clear();
   favs->findPlayer ( playerEdit->text(), exactCheckBox->isChecked(),
      caseCheckBox->isChecked());
   andro->findPlayer ( playerEdit->text(), exactCheckBox->isChecked(), 
      caseCheckBox->isChecked());
}

void SearchDialog::found( QString hostname, QString address, QString name )
{
   // Only add player once if they are on multiple server lists
   if ( playerList->findItem ( name, 0 ) == 0 )
   {
      ServerItem *player = new ServerItem ( playerList );
      player->setText ( 0, name );
      player->setText ( 1, hostname );
      player->setText ( 2, address );
      if ( buddies->isBuddy ( name ))
      {
         QPixmap buddyIcon;
         buddyIcon = QPixmap ( buddy_xpm );
         player->setPixmap ( 0, buddyIcon );
      }
   }
}

void SearchDialog::setListViews ( ServerListView *favourites, 
   ServerListView *andromeda )
{
   favs = favourites;
   andro = andromeda;
   connect ( favs, SIGNAL ( playerFound ( QString, QString, QString )), this,
      SLOT ( found ( QString, QString, QString )));
   connect ( andro, SIGNAL ( playerFound ( QString, QString, QString )), this,
      SLOT ( found ( QString, QString, QString )));
}

void SearchDialog::setBuddyList ( BuddyList *buddyList )
{
   buddies = buddyList;
}

void SearchDialog::playerRMB ( QListViewItem *item, const QPoint &point, int )
{
   QPopupMenu *playerMenu = new QPopupMenu ( this );
   if ( item )
   {
      if ( !item->pixmap ( 0 ))
      {
         playerMenu->insertItem ( "Add player to buddy list", this,
	    SLOT ( addToBuddyList ()));
      }
      playerMenu->insertItem ( "Show server", this, SLOT ( showServer()));
      playerMenu->popup ( point );
   }
}

void SearchDialog::addToBuddyList ()
{
   QPixmap buddyIcon;
   buddyIcon = QPixmap ( buddy_xpm );
   buddies->saveBuddy ( playerList->currentItem()->text (0),
      playerList->currentItem()->text (2), playerList->currentItem()->text (1));
   playerList->currentItem()->setPixmap ( 0, buddyIcon );
}

void SearchDialog::showServer()
{
   emit raiseServer ( playerList->currentItem()->text (2),
      playerList->currentItem()->text (0));
}

//-----------------------------------------------------------------------------
//  Subclass qlistviewitem to allow drawing the colored server names
//-----------------------------------------------------------------------------
ServerItem::ServerItem ( QListView *item )
	: QListViewItem ( item )
{
}

void ServerItem::paintCell ( QPainter *p, const QColorGroup &cg,
   int column, int width, int align )
{
   if ( column == 1 )
   {
      QString name = text ( column );
      if ( name.startsWith ( "<qt>" ))
      {
         QBrush br = cg.brush ( QColorGroup::Base );
	 QSimpleRichText tmp ( name, listView()->font());
	 tmp.draw ( p, 0, -2, QRect ( 0, 0, width, height()), cg, &br );
      }
      else
         QListViewItem::paintCell ( p, cg, column, width, align );
   }
   else
      QListViewItem::paintCell ( p, cg, column, width, align );
}
