// serverlistitem.cpp
//
/****************************************************************************
   Copyright (C) 2004, David Henry. All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*****************************************************************************/
				    
#include "serverlistitem.h"
#include <qpainter.h>
#include <qstring.h>
#include <qpixmap.h>
#include <qsimplerichtext.h>

ServerListItem::ServerListItem ( QListView *parent )
	: QListViewItem ( parent )
{
   QPtrList<Settings> settingList;
   settingList.setAutoDelete (TRUE);
   // numPlayers holds the actual number of players in the data
   numPlayers = 0;
   tour = "";
   type = "";
   timeouts = 0;
}

ServerListItem::~ServerListItem()
{
   Settings *settings;
   for ( settings = settingsList.first(); settings;
        settings = settingsList.next())
      delete settings;
}

//-------------------------------------------------------------------
//  Reimplement QListViewItem's compare function to sort IP's, icons and
//  integers properly
//-------------------------------------------------------------------
int ServerListItem::compare ( QListViewItem *i, int col, bool ascending ) const
{
   QString value1, value2;
   value1 = key ( col, ascending ); 
   value2 = i->key ( col, ascending );
   
   // Sort IP addresses 
   if ( col == 1 )
   {
      QString port1 = value1.section ( ':', 1, 1 );
      QString port2 = value2.section ( ':', 1, 1 );
      value1 = value1.section ( ':', 0, 0 );
      value2 = value2.section ( ':', 0, 0 );
      QString temp1, temp2;
      for ( int i = 0; i < 4; i++ )
      {
         temp1 = value1.section ( '.', i, i );
         temp2 = value2.section ( '.', i, i );
         if ( temp1.toInt() < temp2.toInt())
            return -1;
         if ( temp1.toInt() > temp2.toInt())
            return 1;
      }
      if ( port1.toInt() < port2.toInt())
         return -1;
      if ( port2.toInt() > port2.toInt())
         return 1;
      return 0;
   }
   else if ( col == 2 )
   {
      for ( int i = 0; i < 2; i++ )
      {
         QString num1 = value1.section ( " / ", i, i );
         QString num2 = value2.section ( " / ", i, i );
         if ( num1.toInt() < num2.toInt())
            return -1;
         if ( num1.toInt() > num2.toInt())
            return 1;
      }
      return 0;
   }
   else if ( col == 3 )
   {
      // Check if item2 has timed out
      if ( i->pixmap( 3 ))
         return -1;
      // Check if item1 has timed out
      if ( pixmap ( 3 ))
         return 1;
      if ( value1.isEmpty())
         return 1;
      if ( value2.isEmpty())
         return -1;
      if ( value1.toInt() < value2.toInt())
         return -1;
      if ( value1.toInt() > value2.toInt())
         return 1;
      return 0;
   }
   else if ( col == 5 || col == 6 || col == 7 )
   {
      if ( i->pixmap ( col ))
         return 1;
      if ( pixmap ( col ))
         return -1;
      return 0;
   }
   return value1.compare ( value2 );
}

void ServerListItem::paintCell ( QPainter *p, const QColorGroup &cg,
   int column, int width, int align )
{
   if ( column == 0 )
   {
      QString name = text(column);
      if ( name.startsWith ( "<qt>" ))
      {
         QBrush br;

	 if ( isSelected())
	    br = cg.brush ( QColorGroup::Highlight );
	 else
	    br = cg.brush ( QColorGroup::Base );

         int pos = 1;
	 if ( pixmap ( column ))
	 {
	    const QPixmap *icon = pixmap ( column );
	    // Blank the rectangle first
	    int y0 = ( height() - icon->height()) / 2;
	    p->fillRect ( 0, 0, width, height(), br );
	    p->drawPixmap ( pos, y0, *pixmap(column));
	    pos += pixmap(column)->width() + 2;
	 }
	 QSimpleRichText tmp ( name, listView()->font());
	 tmp.draw ( p, pos, -2, QRect ( pos, 0, width - pos, height()), cg, 
	    &br );
      }
      else
         QListViewItem::paintCell ( p, cg, column, width, align );
   }
   else
      QListViewItem::paintCell ( p, cg, column, width, align );
}

void ServerListItem::addNewSetting ( QString name, QString value )
{
   Settings *settings;
   name = name.remove ( " " );
   for ( settings = settingsList.first(); settings;
      settings = settingsList.next())
   {
      if ( settings->name() == name )
      {
         settings->setValue ( value );
	 return;
      }
   }     
   settingsList.append ( new Settings ( name, value ));
}

QStringList ServerListItem::getSettingNames ()
{
   Settings *settings;
   QStringList nameList;
   for ( settings = settingsList.first(); settings;
      settings = settingsList.next())
      nameList.append ( settings->name());
   return nameList;
   
}

QStringList ServerListItem::getSettingValues ()
{
   Settings *settings;
   QStringList valueList;
   for ( settings = settingsList.first(); settings;
      settings = settingsList.next())
      valueList.append ( settings->value());
   return valueList;
}

void ServerListItem::setPlayerNumber ( int num )
{
   playerList.reserve ( num );
   numPlayers = num;
}

int ServerListItem::getPlayerNumber ( )
{
   return numPlayers;
}

void ServerListItem::setServerType ( QString serverType )
{
   type = serverType;
}

QString ServerListItem::getServerType ( )
{
   return type;
}

void ServerListItem::setServerTour ( QString serverTour )
{
   tour = serverTour;
}

QString ServerListItem::getServerTour ( )
{
   return tour;
}

void ServerListItem::setServerLocation ( QString serverLocation )
{
   location = serverLocation;
}

QString ServerListItem::getServerLocation ( )
{
   return location;
}

void ServerListItem::setServerVersion ( QString serverVersion )
{
   version = serverVersion;
}

QString ServerListItem::getServerVersion ( )
{
   return version;
}

void ServerListItem::setServerContinent ( QString serverCont )
{
   continent = serverCont;
}

QString ServerListItem::getServerContinent ( )
{
   return continent;
}

void ServerListItem::setNumTimeout( int num )
{
   timeouts = num;
}

int ServerListItem::getNumTimeout()
{
   return timeouts;
}

void ServerListItem::setPlayerName ( int num, QString newName )
{
   playerList[num].setName ( newName );
}

void ServerListItem::setPlayerGoal ( int num, QString goal )
{
   playerList[num].setGoal ( goal );
}

void ServerListItem::setPlayerPing ( int num, QString ping )
{
   playerList[num].setPing ( ping );
}

void ServerListItem::setPlayerHonor ( int num, QString honor )
{
   playerList[num].setHonor ( honor );
}

void ServerListItem::setPlayerKia ( int num, QString kia )
{
   playerList[num].setKia ( kia );
}

void ServerListItem::setPlayerRoe ( int num, QString roe )
{
   playerList[num].setRoe ( roe );
}

void ServerListItem::setPlayerLeader ( int num, QString leader )
{
   playerList[num].setLeader ( leader );
}

void ServerListItem::setPlayerEnemy ( int num, QString enemy )
{
   playerList[num].setEnemy ( enemy );
}

void ServerListItem::setPlayerBuddy ( int num, bool buddy )
{
   playerList[num].setBuddy ( buddy );
}

void ServerListItem::setPlayerBuddy ( QString name, bool buddy )
{
   for ( int i = 0; i < numPlayers; i++ )
   {
      if ( playerList[i].name() == name )
      {
         playerList[i].setBuddy ( buddy );
	 return;
      }
   }
}

QString ServerListItem::getPlayerName ( int num )
{
   return playerList[num].name();
}

QString ServerListItem::getPlayerGoal ( int num )
{
   return playerList[num].goal();
}

QString ServerListItem::getPlayerPing ( int num )
{
   return playerList[num].ping();
}

QString ServerListItem::getPlayerHonor ( int num )
{
   return playerList[num].honor();
}

QString ServerListItem::getPlayerKia ( int num )
{
   return playerList[num].kia();
}

QString ServerListItem::getPlayerRoe ( int num )
{
   return playerList[num].roe();
}

QString ServerListItem::getPlayerLeader ( int num )
{
   return playerList[num].leader();
}

QString ServerListItem::getPlayerEnemy ( int num )
{
   return playerList[num].enemy();
}

bool ServerListItem::isPlayerBuddy ( int num )
{
   return playerList[num].buddy();
}
