// serverlistitem.h
//
/****************************************************************************
   Copyright (C) 2004, David Henry. All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*****************************************************************************/
				    

#ifndef SERVERLISTITEM_H
#define SERVERLISTITEM_H

#include <qlistview.h>
#include <qvaluevector.h>

class Settings
{
public:
   Settings ( const QString &name, const QString &value )
      : sName ( name ), sValue ( value )
   {}
   QString name() const { return sName; }
   QString value() const { return sValue; }
   void setValue ( QString v ) { sValue = v; }

private:
   QString sName, sValue;
};

class Player
{
public:
   Player () {}
   Player ( const QString &name, const QString &honor, const QString &leader,
      const QString &goal, const QString &kia, const QString &roe, 
      const QString &enemy, const QString &ping, const bool &buddy )
      : sName ( name ), sHonor ( honor ), sLeader ( leader ), sGoal ( goal ),
        sKia ( kia ), sRoe ( roe ), sEnemy ( enemy ), sPing ( ping ),
        sBuddy ( buddy )
      {}
      QString name() const { return sName; }
      QString honor() const { return sHonor; }
      QString leader() const { return sLeader; }
      QString goal() const { return sGoal; }
      QString kia() const { return sKia; }
      QString roe() const { return sRoe; }
      QString enemy() const { return sEnemy; }
      QString ping() const { return sPing; }
      bool buddy() const { return sBuddy; }
      void setName ( QString n ) { sName = n; }
      void setHonor ( QString h ) { sHonor = h; }
      void setLeader ( QString l ) { sLeader = l; }
      void setGoal ( QString g ) { sGoal = g; }
      void setKia ( QString k ) { sKia = k; }
      void setRoe ( QString r ) { sRoe = r; }
      void setEnemy ( QString e ) { sEnemy = e; }
      void setPing ( QString p ) { sPing = p; }
      void setBuddy ( bool b ) { sBuddy = b; }

private:
   QString sName, sHonor, sLeader, sGoal, sKia, sRoe, sEnemy, sPing;
   bool sBuddy;
};

class QPainter;
class QColorGroup;
   
class ServerListItem: public QListViewItem
{
public:
   ServerListItem ( QListView *parent );
   ~ServerListItem ();
   void addNewSetting ( QString, QString );
   void paintCell ( QPainter *, const QColorGroup &, int, int, int );
   QStringList getSettingNames ();
   QStringList getSettingValues ();
   void setPlayerNumber ( int );
   int getPlayerNumber ();
   void setServerType ( QString );
   QString getServerType ( );
   void setServerTour ( QString );
   QString getServerTour ( );
   void setServerLocation ( QString );
   QString getServerLocation ( );
   void setServerVersion ( QString );
   QString getServerVersion ();
   void setServerContinent ( QString );
   QString getServerContinent ();
   void setNumTimeout( int);
   int getNumTimeout();
   void setPlayerName ( int, QString );
   void setPlayerGoal ( int, QString );
   void setPlayerPing ( int, QString );
   void setPlayerHonor ( int, QString );
   void setPlayerKia ( int, QString );
   void setPlayerRoe ( int, QString );
   void setPlayerLeader ( int, QString );
   void setPlayerEnemy ( int, QString );
   void setPlayerBuddy ( int, bool );
   void setPlayerBuddy ( QString, bool );
   QString getPlayerName ( int );
   QString getPlayerGoal ( int );
   QString getPlayerPing ( int );
   QString getPlayerHonor ( int );
   QString getPlayerKia ( int );
   QString getPlayerRoe ( int );
   QString getPlayerLeader ( int );
   QString getPlayerEnemy ( int );
   bool isPlayerBuddy ( int );

private:
   int compare ( QListViewItem *, int, bool ) const;

   int numPlayers;
   int timeouts;
   QString tour;
   QString type;
   QString location;
   QString version;
   QString continent;
   QPtrList<Settings> settingsList;
   QValueVector<Player> playerList;
};

#endif
