// serverlistview.cpp
//
/****************************************************************************
   Copyright (C) 2004, David Henry. All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*****************************************************************************/

#include "config.h"

#include "serverlistview.h"
#include "buddylist.h"
#include "serverlistitem.h"

#include <qdom.h>
#include <qlistview.h>
#include <qpixmap.h>
#include <qdir.h>

#include "../xpms/buddy.xpm"
#include "../xpms/button_cancel.xpm"
#include "../xpms/honor.xpm"
#include "../xpms/leased.xpm"
#include "../xpms/miles.xpm"
#include "../xpms/official.xpm"
#include "../xpms/private.xpm"
#include "../xpms/punkbuster.xpm"
#include "../xpms/tour1.xpm" 
#include "../xpms/tour2.xpm"
#include "../xpms/tour3.xpm" 
#include "../xpms/tour6.xpm"
#include "../xpms/tour7.xpm"
#include "../xpms/tour8.xpm"
#include "../xpms/tour11.xpm"

#ifdef USE_GEOIP
#include <GeoIP.h>
#endif

const QString SETTINGS_DIR = "/.aabrowse/";
const QString packageDataDir = PACKAGE_DATA_DIR;

ServerListView::ServerListView ( QWidget *parent )
   : QSplitter ( parent )
{
   setOrientation ( QSplitter::Vertical );
   serverList = new QListView ( this );

   QSplitter *varSplit = new QSplitter ( Qt::Horizontal, this );
   playerList = new QListView ( varSplit );
   settingsList = new QListView ( varSplit );

   serverList->setAllColumnsShowFocus ( TRUE );
   serverList->setSelectionMode ( QListView::Extended );
   serverList->setShowSortIndicator ( TRUE );
   serverList->addColumn ( "Name", 300 );
   serverList->addColumn ( "Address", 200 );
   serverList->addColumn ( "Players" );
   serverList->addColumn ( "Ping" );
   serverList->addColumn ( "Map" );
   serverList->addColumn ( "PB" );
   serverList->addColumn ( "Miles" );
   serverList->addColumn ( "Private" );
   serverList->setSorting ( 3 );
   
   settingsList->setShowSortIndicator ( TRUE );
   settingsList->addColumn ( "Setting" );
   settingsList->addColumn ( "Value" );

   playerList->setShowSortIndicator ( TRUE );
   playerList->addColumn ( "Name" );
   playerList->addColumn ( "Honor" );
   playerList->addColumn ( "Score" );
   playerList->addColumn ( "Leader" );
   playerList->addColumn ( "Kills" );
   playerList->addColumn ( "KIA" );
   playerList->addColumn ( "ROE" );
   playerList->addColumn ( "Ping" );

   playerCount = 0;
   serverCount = 0;
   colouredNames = TRUE;

   list.resize ( 2003 );

   connect ( serverList, SIGNAL ( clicked ( QListViewItem * )),
      SLOT ( clicked ( QListViewItem * )));
   connect ( serverList, SIGNAL ( doubleClicked ( QListViewItem *,
      const QPoint&, int )), SLOT ( doubleClicked ( QListViewItem *,
      const QPoint&, int )));
   connect ( serverList, SIGNAL ( contextMenuRequested ( QListViewItem *,
      const QPoint&, int )), SLOT ( rightClickServer ( QListViewItem *,
      const QPoint&, int )));
   connect ( playerList, SIGNAL ( contextMenuRequested ( QListViewItem *,
      const QPoint&, int )), SLOT ( rightClickPlayer ( QListViewItem *,
      const QPoint&, int )));
}

//-------------------------------------------------------------------
//  Adds new server to serverlist. Sets currentServer to point to new server
//-------------------------------------------------------------------
void ServerListView::addNewServer ()
{
   ServerListItem *newServer = new ServerListItem ( serverList );
   serverCount++;
   currentServer = newServer;
}

//-------------------------------------------------------------------
//  Adds a new server and sets the address only if this server does
//  not exist already in the list. Returns true if server is added
//-------------------------------------------------------------------
bool ServerListView::addNewServer ( QString address )
{
   QListViewItem *item = list[address];
   if ( !item )
   {
      ServerListItem *newServer = new ServerListItem ( serverList );
      serverCount++;
      currentServer = newServer;
      setServerAddress ( address );
      return TRUE;
   }
   return FALSE;
}

//-------------------------------------------------------------------
//  Deletes the currently highlighted server
//-------------------------------------------------------------------
void ServerListView::deleteCurrentServer ()
{
   if ( serverList->currentItem())
   {
      playerCount -= serverList->currentItem()->text(2).section 
         ( " / ", 0, 0 ).toInt();
      serverCount--;
      list.remove ( serverList->currentItem()->text(1));
      serverList->currentItem()->~QListViewItem();
   }
}

void ServerListView::setBuddyList ( BuddyList *list )
{
   buddyList = list;
}

void ServerListView::findPlayer ( QString player, bool exact, bool caseSens )
{
   QListViewItemIterator it ( serverList );
   ServerListItem *item;
   while ( it.current())
   {
      item = dynamic_cast<ServerListItem*>(it.current());
      for ( int i = 0; i < item->getPlayerNumber (); i++ )
      {
         QString playerName = item->getPlayerName ( i );
         if ( exact )
	 {
	    if ( !caseSens )
	    {
	       playerName = playerName.lower();
	       player = player.lower();
	    }
	    if ( playerName == player )
	       emit playerFound ( item->text ( 0 ), item->text ( 1 ),
	          playerName );
	 }
	 else
	 {
	    if ( playerName.find ( player, 0, caseSens ) >= 0 )
	       emit playerFound ( item->text ( 0 ), item->text ( 1 ),
	          playerName );
         }
      }
      it++;
   }
}

int ServerListView::getPlayerCount ()
{
   return playerCount;
}

int ServerListView::getServerCount ()
{
   return serverCount;
}

void ServerListView::setServerCount ( int count )
{
   serverCount = count;
}

void ServerListView::setPlayerCount ( int count )
{
   playerCount = count;
}

//-------------------------------------------------------------------
//  Sets the timeout for the currently highlighted server. Returns the
//  number of times server has timed out
//-------------------------------------------------------------------
void ServerListView::setTimeout ( QString address )
{
   ServerListItem *item = list[address];
   if ( item )
   {
      QPixmap timeoutIcon;
      timeoutIcon = QPixmap ( button_cancel_xpm );
      item->setPixmap ( 3, timeoutIcon );
      item->setText ( 3, "" );
      // Reset player counts
      item->setText ( 2, "" );
      item->setPixmap ( 2, QPixmap::QPixmap());
      item->setPlayerNumber ( 0 );
   }
}

int ServerListView::getTimeoutNum ( QString address )
{
   ServerListItem *item = list[address];
   if ( item )
   {
      item->setNumTimeout( item->getNumTimeout() + 1);
      return item->getNumTimeout();
   }
   return -1;
}

//-------------------------------------------------------------------
//  Sets currentServer to point to the server "address"
//-------------------------------------------------------------------
void ServerListView::setCurrentServer ( QString address )
{
   currentServer = list[address];
}

bool ServerListView::isCurrentServerSet ()
{
   if ( currentServer )
      return TRUE;
   else return FALSE;
}

//-------------------------------------------------------------------
//  The server pointed to by currentServer is highlighted
//-------------------------------------------------------------------
void ServerListView::selectCurrentServer ()
{
   currentServer->setSelected ( TRUE );
   serverList->ensureItemVisible ( currentServer );
}

//-------------------------------------------------------------------
//
//  Set server values for currentServer
//
//-------------------------------------------------------------------
void ServerListView::setServerPing ( int time )
{
   currentServer->setPixmap ( 3, QPixmap::QPixmap());
   currentServer->setText ( 3, QString::number ( time ));
}

void ServerListView::setServerAddress ( QString address )
{
   list.insert ( address, currentServer );
   currentServer->setText ( 1, address );
   // Set location variable for filtering
   if ( address.startsWith ( "64.74" ))
   {
      int num = address.section ( ".", 2, 2 ).toInt();
      if ( num >= 136 && num <= 139 )
         currentServer->setServerLocation ( "West Coast" );
   }
   else if ( address.startsWith ( "66.117.43" ))
      currentServer->setServerLocation ( "West Coast" );
   else if ( address.startsWith ( "64.167.48" ))
      currentServer->setServerLocation ( "West Coast" );
   else if ( address.startsWith ( "69.25" ))
   {
      int num = address.section ( ".", 2, 2 ).toInt();
      if ( num >= 16 && num <= 19 || num >= 22 && num <= 23 )
         currentServer->setServerLocation ( "East Coast" );
   }
   else if ( address.startsWith ( "213.228.238" ) ||
         address.startsWith ( "213.228.239" ))
      currentServer->setServerLocation ( "Euro" );
   if ( currentServer->getServerLocation ().isEmpty())
      currentServer->setServerLocation ( "Other" );
#ifdef USE_GEOIP
   GeoIP *gi;
   gi = GeoIP_new(GEOIP_STANDARD);
   if ( gi == NULL )
      return;

   int id = GeoIP_country_id_by_addr ( gi, address.section ( ":", 0, 0 ));
   QString code = GeoIP_country_code[id];
   QString continent = GeoIP_country_continent[id];
   GeoIP_delete ( gi );
   if ( code.isEmpty())
      return;

   code = packageDataDir + "/flags/" + code.lower().append ( ".png" );
   QPixmap country;
   country = QPixmap  ( code );
   currentServer->setPixmap ( 1, country );
   if ( continent == "OC" )
      currentServer->setServerContinent ( "Oceania" );
   else if ( continent == "AF" )
      currentServer->setServerContinent ( "Africa" );
   else if ( continent == "AS" )
      currentServer->setServerContinent ( "Asia" );
   else if ( continent == "EU" )
      currentServer->setServerContinent ( "Europe" );
   else if ( continent == "NA" )
      currentServer->setServerContinent ( "North America" );
   else if ( continent == "SA" )
      currentServer->setServerContinent ( "South America" );
#endif
}

void ServerListView::updateCurrentPing ( int time )
{
   serverList->currentItem()->setText ( 3, QString::number ( time ));
   serverList->currentItem()->setPixmap ( 3, QPixmap::QPixmap());
}

void ServerListView::setColouredNames ( bool b )
{
   colouredNames = b;
}

void ServerListView::setServerName ( QString name )
{
   if ( name.contains ( 0x1b ))
   {
      if ( colouredNames )
      {
         // Convert colour info to rich text format
         name.prepend ( "<qt><nobr>" );
         bool trailing = FALSE;
         while ( name.contains ( 0x1b ))
         {
            int pos = name.find ( 0x1b );
            unsigned char r, g, b;
            r = name[pos+1].latin1();
            g = name[pos+2].latin1();
            b = name[pos+3].latin1();
            name.replace ( pos, 4, QString ( "<font color=\"%1\">" ).arg (
               QColor ( r, g, b ).name()));
            if ( trailing )
               name.insert ( pos, "</font>" );
            trailing = TRUE;
         }
         name.append ( "</font></nobr></qt>" );
      }
      else
      {
	 // Remove all the colour information
         while ( name.contains ( 0x1b ))
	 {
	    int pos = name.find ( 0x1b );
	    name.remove ( pos, 4 );
	 }
      }
   }
   currentServer->setText ( 0, name );
}

void ServerListView::setServerMaxPlayers ( QString maxPlayers )
{
   if ( !maxPlayers.isEmpty())
   {
      QString currentPlayers = currentServer->text ( 2 ).section ( 
         " / ", 0, 0 );
      currentServer->setText ( 2, currentPlayers + " / " + maxPlayers );
   }
}

void ServerListView::setServerNumPlayers ( QString numPlayers )
{
   // This only sets the display of player numbers in the listview.
   // It doesn't affect the number of actual players displayerd
   if ( !numPlayers.isEmpty())
   {
      // Minus old number of players from player count
      int oldNumPlayers = currentServer->text ( 2 ).section ( 
         " / ", 0, 0 ).toInt();
      setPlayerCount ( getPlayerCount() - oldNumPlayers );
      QString maxPlayers = currentServer->text ( 2 ).section ( " / ", 1, 1 );
      currentServer->setText ( 2, numPlayers + " / " + maxPlayers );
      // Add new players to player count
      setPlayerCount ( getPlayerCount() + numPlayers.toInt());
   }
}

void ServerListView::setServerMap ( QString map )
{
   currentServer->setText ( 4, map );
}

void ServerListView::setServerTour ( QString tour )
{
   QPixmap tourIcon;
   if ( tour == "1" )
      tourIcon = QPixmap ( tour1_xpm );
   if ( tour == "2" )
      tourIcon = QPixmap ( tour2_xpm );
   if ( tour == "3" )
      tourIcon = QPixmap ( tour3_xpm );
   if ( tour == "6" )
      tourIcon = QPixmap ( tour6_xpm );
   if ( tour == "7" )
      tourIcon = QPixmap ( tour7_xpm );
   if ( tour == "8" )
      tourIcon = QPixmap ( tour8_xpm );
   if ( tour == "11" || tour == "12" )
      tourIcon = QPixmap ( tour11_xpm );
   currentServer->setPixmap ( 4, tourIcon );
   currentServer->setServerTour ( tour );
}

void ServerListView::setServerMiles ( bool miles )
{
   if ( miles )
   {
      QPixmap milesIcon;
      milesIcon = QPixmap ( miles_xpm );
      currentServer->setPixmap ( 6, milesIcon );
   }
   else
      currentServer->setPixmap ( 6, QPixmap::QPixmap());
}

void ServerListView::setServerOfficial ( bool official )
{
   if ( official )
   {
      QPixmap serverIcon;
      serverIcon = QPixmap ( official_xpm );
      currentServer->setPixmap ( 0, serverIcon );
      currentServer->setServerType ( "Official" );
   }
   else
   {
      currentServer->setPixmap ( 0, QPixmap::QPixmap());
      currentServer->setServerType ( "Authorized" );
   }
}

void ServerListView::setServerHonor ( bool honor )
{
   if ( honor )
   {
      QPixmap honorIcon;
      honorIcon = QPixmap ( honor_xpm );
      currentServer->setPixmap ( 0, honorIcon );
      currentServer->setServerType ( "Leased Official" );
   }
   else
   {
      currentServer->setPixmap ( 0, QPixmap::QPixmap());
      currentServer->setServerType ( "Authorized" );
   }
}

void ServerListView::setServerLeased ( bool leased )
{
   if ( leased )
   {
      QPixmap leasedIcon;
      leasedIcon = QPixmap ( leased_xpm );
      currentServer->setPixmap ( 0, leasedIcon );
      currentServer->setServerType ( "Leased" );
   }
   else
   {
      currentServer->setPixmap ( 0, QPixmap::QPixmap());
      currentServer->setServerType ( "Authorized" );
   }
}

void ServerListView::setServerPrivate ( bool locked )
{
   if ( locked )
   {
      QPixmap privateIcon;
      privateIcon = QPixmap ( private_xpm );
      currentServer->setPixmap ( 7, privateIcon );
   }
   else
      currentServer->setPixmap ( 7, QPixmap::QPixmap());
}

void ServerListView::setServerPB ( bool pb )
{
   if ( pb )
   {
      QPixmap pbIcon;
      pbIcon = QPixmap ( punkbuster_xpm );
      currentServer->setPixmap ( 5, pbIcon );
   }
   else
      currentServer->setPixmap ( 5, QPixmap::QPixmap());
}

void ServerListView::setPlayerValue ( int playerNum, QString type,
   QString value )
{
   if ( type == "player" )
   {
      currentServer->setPlayerName ( playerNum, value );
      if ( !value.isEmpty() && buddyList->isBuddy ( value ))
      {
         setServerBuddyIcon ( TRUE );
         currentServer->setPlayerBuddy ( playerNum, TRUE );
         buddyList->updateBuddy ( value, currentServer->text(1),
            currentServer->text(0));
      }
      else
         currentServer->setPlayerBuddy ( playerNum, FALSE );
   }
   else if ( type == "honor" )
      currentServer->setPlayerHonor ( playerNum, value );
   else if ( type == "goal" )
      currentServer->setPlayerGoal ( playerNum, value );
   else if ( type == "leader" )
      currentServer->setPlayerLeader ( playerNum, value );
   else if ( type == "enemy" )
      currentServer->setPlayerEnemy ( playerNum, value );
   else if ( type == "kia" )
      currentServer->setPlayerKia ( playerNum, value );
   else if ( type == "roe" )
      currentServer->setPlayerRoe ( playerNum, value );
   else if ( type == "ping" )
      currentServer->setPlayerPing ( playerNum, value );
}

void ServerListView::setSettingValue ( QString type, QString value )
{
   if ( type == "Version" )
      currentServer->setServerVersion ( value );
   currentServer->addNewSetting ( type, value );
}

//-------------------------------------------------------------------
//  Adds a buddy icon to the currently highlighted player name
//-------------------------------------------------------------------
void ServerListView::setPlayerBuddyIcon ( bool buddy )
{
   ServerListItem *item = 
      dynamic_cast<ServerListItem*>(serverList->currentItem());
   if ( buddy )
   {
      QPixmap buddyIcon;
      buddyIcon = QPixmap ( buddy_xpm );
      playerList->currentItem()->setPixmap ( 0, buddyIcon );
      item->setPixmap ( 2, buddyIcon );
   }
   else
      playerList->currentItem()->setPixmap ( 0, QPixmap::QPixmap());

   item->setPlayerBuddy ( playerList->currentItem()->text(0), buddy );
}

//-------------------------------------------------------------------
//  Adds a buddy icon to the player column of currentServer
//-------------------------------------------------------------------
void ServerListView::setServerBuddyIcon ( bool buddy )
{
   if ( buddy )
   {
      QPixmap buddyIcon;
      buddyIcon = QPixmap ( buddy_xpm );
      currentServer->setPixmap ( 2, buddyIcon );
   }
   else
      currentServer->setPixmap ( 2, QPixmap::QPixmap());
}

void ServerListView::setActualPlayerNumber ()
{
   // This sets the number of players we have real data for
   currentServer->setPlayerNumber ( currentServer->text(2).section ( 
      " / ", 0,0 ).toInt());
}

//-------------------------------------------------------------------
//   Mouse click signals
//-------------------------------------------------------------------
void ServerListView::clicked ( QListViewItem *thisItem )
{
   emit serverClicked ( thisItem );
   // Do nothing if no item is clicked
   if ( !thisItem  )
      return;
   updatePlayersSettings ();
}

void ServerListView::updatePlayersSettings ()
{
   // Update the settings listview
   ServerListItem *item = dynamic_cast<ServerListItem*>(serverList->currentItem());
   clearPlayerSettingLists();
   QStringList names = item->getSettingNames();
   QStringList values = item->getSettingValues();
   
   QStringList::Iterator itNames = names.begin();
   QStringList::Iterator itValues = values.begin();
   while ( itNames != names.end() && itValues != values.end() )
   {
      QListViewItem *newItem = new QListViewItem ( settingsList );
      newItem->setText ( 0, *itNames );
      newItem->setText ( 1, *itValues );
      itNames++;
      itValues++;
   }
   // Update player listview
   
   for ( int i = 0; i < item->getPlayerNumber(); i++ )
   {
      QListViewItem *newItem = new QListViewItem ( playerList );
      newItem->setText ( 0, item->getPlayerName ( i ));
      newItem->setText ( 1, item->getPlayerHonor ( i ));
      newItem->setText ( 2, item->getPlayerGoal ( i ));
      newItem->setText ( 3, item->getPlayerLeader ( i ));
      newItem->setText ( 4, item->getPlayerEnemy ( i ));
      newItem->setText ( 5, item->getPlayerKia ( i ));
      newItem->setText ( 6, item->getPlayerRoe ( i ));
      newItem->setText ( 7, item->getPlayerPing ( i ));
      if ( item->isPlayerBuddy ( i ))
      {
         QPixmap buddyIcon;
         buddyIcon = QPixmap ( buddy_xpm );
         newItem->setPixmap ( 0, buddyIcon );
      }
   }
}

void ServerListView::doubleClicked ( QListViewItem *item, const QPoint& point,
   int pos )
{
   emit serverDoubleClicked ( item, point, pos );
}

void ServerListView::rightClickPlayer ( QListViewItem *item, const QPoint& point,
   int pos )
{
   emit playerRightClick ( item, point, pos );
}

void ServerListView::rightClickServer ( QListViewItem *item, const QPoint& point,
   int pos )
{
   emit serverRightClick ( item, point, pos );
}

//-------------------------------------------------------------------
//  Returns a list of all visible servers in the listview
//-------------------------------------------------------------------
QStringList ServerListView::getServerAddressList ()
{
   QStringList address;
   QListViewItemIterator it ( serverList );
   while ( it.current())
   {
      if ( it.current()->isVisible())
      {
	 // Reset the number of timeouts as servers are added
         ServerListItem *item = dynamic_cast<ServerListItem*>(it.current());
	 item->setNumTimeout ( 0 );
         address.append ( item->text(1));
      }
      ++it;
   }
   return address;
}

//-------------------------------------------------------------------
//  Returns a list of all visible and selected servers in the listview
//-------------------------------------------------------------------
QStringList ServerListView::getSelectedAddressList ()
{
   QStringList address;
   QListViewItemIterator it ( serverList, QListViewItemIterator::Selected );
   while ( it.current())
   {
      if ( it.current()->isVisible())
      {
         // Reset the number of timeouts as servers are added
         ServerListItem *item = dynamic_cast<ServerListItem*>(it.current());
	 item->setNumTimeout ( 0 );
	 address.append ( item->text(1));
      }
      ++it;
   }
   return address;
}

void ServerListView::clearServerList()
{
   if ( serverList->childCount())
   {
      list.clear();
      serverList->clear();
      clearPlayerSettingLists();
   }
}

void ServerListView::clearPlayerSettingLists()
{
   if ( playerList->childCount())
      playerList->clear();
   if ( settingsList->childCount())
      settingsList->clear();
}

void ServerListView::removeTimedoutServers()
{
   QListViewItemIterator it ( serverList );
   while ( it.current() )
   {
      if ( it.current()->pixmap ( 3 ) != 0 )
      {
         serverCount--;
	 list.remove ( it.current()->text(1));
         it.current()->~QListViewItem();
      }
      else
         it++;
   }
}

//-------------------------------------------------------------------
//
//  Return values for the currently highlighted server
//
//-------------------------------------------------------------------
QString ServerListView::getCurrentName ()
{
   return serverList->currentItem()->text(0);
}

QString ServerListView::getCurrentAddress ()
{
   if ( serverList->currentItem())
      return serverList->currentItem()->text(1);
   return QString::null;
}

int ServerListView::getCurrentPlayers()
{
   if ( serverList->currentItem())
      return serverList->currentItem()->text(2).section ( " / ", 0, 0).toInt();
   return -1;
}

int ServerListView::getCurrentMaxPlayers()
{
   if ( serverList->currentItem())
      return serverList->currentItem()->text(2).section ( " / ", 1, 1).toInt();
   return -1;
}

QString ServerListView::getCurrentPlayerName()
{
   if ( playerList->currentItem())
      return playerList->currentItem()->text(0);
   return QString::null;
}

//-------------------------------------------------------------------
//  Set servers invisible if they contain certain values
//-------------------------------------------------------------------
void ServerListView::setServerVisible ( QString key, QStringList match,
   bool state )
{
   playerCount=0, serverCount=0;
   QListViewItemIterator it ( serverList );
   int column=0;
   if ( key == "Map" )
      column = 4;
   else if ( key == "Server Type" )
      column = 0;
   else if ( key == "Location" )
      column = 1;
   else if ( key == "Version" )
      column = 2;
   else if ( key == "Ping" )
      column = 3;
   else if ( key == "Country" )
      column = 5;
      
   while ( it.current())
   {
      bool hasMatched = FALSE;
      for ( QStringList::Iterator listIt = match.begin(); listIt!=match.end();
         ++listIt)
      {
         if ( column == 0 )
	 {
	    ServerListItem *item = dynamic_cast<ServerListItem*>(it.current());
	    if ( item->getServerType() == *listIt )
	       hasMatched = TRUE;
	 }
	 if ( column == 1 )
	 {
            ServerListItem *item = dynamic_cast<ServerListItem*>(it.current());
	    if ( item->getServerLocation () == *listIt )
	       hasMatched = TRUE;
	 }
	 else if ( column == 2 )
	 {
	    ServerListItem *item = dynamic_cast<ServerListItem*>(it.current());
	    if ( item->getServerVersion () == *listIt )
	       hasMatched = TRUE;
	 }
	 else if ( column == 3 )
	 {
	    if ( *listIt == "Timeout" )
	    {
	       if ( it.current()->pixmap( 3 ))
	          hasMatched = TRUE;
            }
            else
	    {
	       int upper = (*listIt).section ( " - ", 1, 1 ).toInt();
	       int lower = (*listIt).section ( " - ", 0, 0 ).toInt();
	       int value = it.current()->text ( 3 ).toInt();
	       if ( upper !=0 && lower !=0 && value <= upper && value >= lower )
		  hasMatched = TRUE;
	       else if ( upper == 0 && lower == 0 && value >= 500 )
	          hasMatched = TRUE;
            }
         }
	 else if ( column == 5 )
	 {
	    ServerListItem *item = dynamic_cast<ServerListItem*>(it.current());
	    if ( item->getServerContinent() == *listIt )
	       hasMatched = TRUE;
	 }
	 else 
	 {
	    QString serverMap = it.current()->text ( column ).lower();
	    QString listMap = (*listIt).lower();
	    if ( listMap == "jrtc farm raid" )
	    {
	       // Workaround for servers that have JRTC Farm set wrong
	       if ( !serverMap.isEmpty() && listMap.contains ( serverMap ))
	          hasMatched = TRUE;
	    }
	    if ( serverMap == listMap )
	       hasMatched = TRUE;
	 }
      }
      if ( hasMatched )
         it.current()->setVisible ( state );
      if ( it.current()->isVisible ())
      {
         serverCount++;
	 playerCount+= it.current()->text (2).section ( " / ", 0, 0).toInt();
      }
      it++;
   }
}

void ServerListView::setServerVisible ( bool value )
{
   serverCount = 0;
   playerCount = 0;
   QListViewItemIterator it ( serverList );
   while ( it.current())
   {
      it.current()->setVisible ( value );
      serverCount++;
      playerCount+= it.current()->text (2).section ( " / ", 0, 0).toInt();
      it++;
   }
}

bool ServerListView::isServerSelected ()
{
   if ( serverList->currentItem())
      return TRUE;
   else
      return FALSE;
}

//----------------------------------------------------------------------------
//  Highlights and shows the named server and shows its playerlist and settings
//  Returns true if found, false otherwise
//----------------------------------------------------------------------------
bool ServerListView::showServer ( QString serverAddress, QString playerName )
{
   QListViewItem *item = list[serverAddress];
   if ( item )
   {
      item->setVisible ( TRUE );
      serverList->clearSelection();
      serverList->ensureItemVisible ( item );
      serverList->setCurrentItem ( item );
      serverList->setSelected ( item, TRUE );
      updatePlayersSettings();
      QListViewItem *player = playerList->findItem ( playerName, 0 );
      if ( player )
      {
	 playerList->clearSelection();
         playerList->ensureItemVisible ( player );
         playerList->setSelected ( player, TRUE );
      }
      emit serverClicked ( item );
      return TRUE;
   }
   return FALSE;
}

//-------------------------------------------------------------------
//  Write an xml file containing all server data to filename
//-------------------------------------------------------------------
void ServerListView::saveServerData ( QString fileName )
{
   QFile dataFile ( QDir::homeDirPath() + SETTINGS_DIR + fileName );
   QTextStream stream ( &dataFile );
   QDomDocument doc ( "Serverdata" );
   QDomElement root = doc.createElement ( "Servers" );
   doc.appendChild ( root );
   QListViewItemIterator it ( serverList );
   while ( it.current())
   {
      ServerListItem *item = dynamic_cast<ServerListItem*>(it.current());
      QDomElement newServer = doc.createElement ( "Server" );
      root.appendChild ( newServer );

      QDomElement hostElement = doc.createElement ( "Hostname" );
      newServer.appendChild ( hostElement );
      QDomText hostText = doc.createTextNode ( item->text (0));
      hostElement.appendChild ( hostText );

      QDomElement addressElement = doc.createElement ( "Address" );
      newServer.appendChild ( addressElement );
      QDomText addressText = doc.createTextNode ( item->text (1));
      addressElement.appendChild ( addressText );

      QDomElement maxPlayersElement = doc.createElement ( "MaxPlayers" );
      newServer.appendChild ( maxPlayersElement );
      QDomText maxPlayersText = doc.createTextNode ( 
         item->text(2).section ( " / ", 1, 1 ));
      maxPlayersElement.appendChild ( maxPlayersText );

      QDomElement currPlayersElement = doc.createElement ( "CurrentPlayers" );
      newServer.appendChild ( currPlayersElement );
      QDomText currPlayersText = doc.createTextNode (
         item->text(2).section ( " / ", 0, 0 ));
      currPlayersElement.appendChild ( currPlayersText );

      QDomElement mapElement = doc.createElement ( "Map" );
      newServer.appendChild ( mapElement );
      QDomText mapText = doc.createTextNode ( item->text (4));
      mapElement.appendChild ( mapText );

      QDomElement tourElement = doc.createElement ( "Tour" );
      newServer.appendChild ( tourElement );
      QDomText tourText = doc.createTextNode ( item->getServerTour ());
      tourElement.appendChild ( tourText );

      QDomElement typeElement = doc.createElement ( "Type" );
      newServer.appendChild ( typeElement );
      QDomText typeText = doc.createTextNode ( item->getServerType ());
      typeElement.appendChild ( typeText );

      QDomElement PBElement = doc.createElement ( "Punkbuster" );
      newServer.appendChild ( PBElement );
      QDomText PBText;
      if ( item->pixmap ( 5 ))
         PBText = doc.createTextNode ( "True" );
      else
         PBText = doc.createTextNode ( "False" );
      PBElement.appendChild ( PBText );

      QDomElement milesElement = doc.createElement ( "Miles" );
      newServer.appendChild ( milesElement );
      QDomText milesText;
      if ( item->pixmap ( 6 ))
         milesText = doc.createTextNode ( "True" );
      else
         milesText = doc.createTextNode ( "False" );
      milesElement.appendChild ( milesText );

      QDomElement privateElement = doc.createElement ( "Private" );
      newServer.appendChild ( privateElement );
      QDomText privateText;
      if ( item->pixmap ( 7 ))
         privateText = doc.createTextNode ( "True" );
      else
         privateText = doc.createTextNode ( "False" );
      privateElement.appendChild ( privateText );

      QStringList nameList = item->getSettingNames();
      QStringList valueList = item->getSettingValues();
      QStringList::Iterator itNames = nameList.begin();
      QStringList::Iterator itValues = valueList.begin();
      while ( itNames != nameList.end() && itValues != valueList.end() )
      {
         QDomElement settingElement = doc.createElement ( 
	    (*itNames).remove ( " " ));
         newServer.appendChild ( settingElement );
         QDomText settingText = doc.createTextNode ( *itValues);
         settingElement.appendChild ( settingText );
         itValues++;
         itNames++;
      }	      
      int playerNum = item->getPlayerNumber();
      if ( playerNum > 0 )
      {
         QDomElement playersElement = doc.createElement ( "Players" );
	 newServer.appendChild ( playersElement );
         for ( int i = 0; i < item->getPlayerNumber(); i++ )
         {
	    QDomElement playerElement = doc.createElement ( "Player" );
	    playersElement.appendChild ( playerElement );
	    
            QDomElement nameElement = doc.createElement ( "Name" );
	    playerElement.appendChild ( nameElement );
	    QDomText nameText = doc.createTextNode ( item->getPlayerName ( i ));
	    nameElement.appendChild ( nameText );

	    QDomElement honorElement = doc.createElement ( "Honor" );
	    playerElement.appendChild ( honorElement );
	    QDomText honorText = doc.createTextNode ( item->getPlayerHonor (i));
	    honorElement.appendChild ( honorText );

	    QDomElement goalElement = doc.createElement ( "Goal" );
	    playerElement.appendChild ( goalElement );
	    QDomText goalText = doc.createTextNode ( item->getPlayerGoal ( i ));
	    goalElement.appendChild ( goalText );

	    QDomElement leaderElement = doc.createElement ( "Leader" );
	    playerElement.appendChild ( leaderElement );
	    QDomText leaderText = doc.createTextNode (
	       item->getPlayerLeader ( i ));
            leaderElement.appendChild ( leaderText );

	    QDomElement enemyElement = doc.createElement ( "Enemy" );
	    playerElement.appendChild ( enemyElement );
	    QDomText enemyText = doc.createTextNode ( item->getPlayerEnemy(i));
	    enemyElement.appendChild ( enemyText );

	    QDomElement kiaElement = doc.createElement ( "Kia" );
	    playerElement.appendChild ( kiaElement );
	    QDomText kiaText = doc.createTextNode ( item->getPlayerKia ( i ));
	    kiaElement.appendChild ( kiaText );

	    QDomElement roeElement = doc.createElement ( "Roe" );
	    playerElement.appendChild ( roeElement );
	    QDomText roeText = doc.createTextNode ( item->getPlayerRoe ( i ));
	    roeElement.appendChild ( roeText );

	    QDomElement pingElement = doc.createElement ( "Ping" );
	    playerElement.appendChild ( pingElement );
	    QDomText pingText = doc.createTextNode ( item->getPlayerPing ( i ));
	    pingElement.appendChild ( pingText );
	 }
      }
      it++;
   }
   if ( dataFile.open ( IO_WriteOnly ))
   {
      stream << doc.toString();
      dataFile.close();
   }
}

void ServerListView::readServerData ( QString fileName )
{
   QFile dataFile ( QDir::homeDirPath() + SETTINGS_DIR + fileName );
   if ( dataFile.open ( IO_ReadOnly ))
   {
      QDomDocument doc ( "serverdata" );
      doc.setContent ( &dataFile );
      QDomNode node = doc.firstChild();
      node = node.firstChild();
      while ( !node.isNull())
      {
         if ( node.isElement() && node.nodeName() == "Server" )
	 {
	    addNewServer ();
	    QDomNode serverNode = node.firstChild();
	    while ( !serverNode.isNull() && serverNode.isElement())
	    {
	       if ( serverNode.nodeName() == "Hostname" )
	          setServerName ( serverNode.toElement().text());
	       else if ( serverNode.nodeName() == "Address" )
	          setServerAddress ( serverNode.toElement().text());
	       else if ( serverNode.nodeName() == "MaxPlayers" )
	          setServerMaxPlayers ( serverNode.toElement().text());
	       else if ( serverNode.nodeName() == "CurrentPlayers" )
	          setServerNumPlayers ( serverNode.toElement().text());
	       else if ( serverNode.nodeName() == "Map" )
	          setServerMap ( serverNode.toElement().text());
	       else if ( serverNode.nodeName() == "Tour" )
	          setServerTour ( serverNode.toElement().text());
	       else if ( serverNode.nodeName() == "Type" )
	       {
	          QString value = serverNode.toElement().text();
		  if ( value == "Official" )
		     setServerOfficial ( TRUE );
		  else if ( value == "Leased Official" )
		     setServerHonor ( TRUE );
		  else if ( value == "Leased" )
		     setServerLeased ( TRUE );
	       }
	       else if ( serverNode.nodeName() == "Miles" )
	       {
	          if ( serverNode.toElement().text() == "True" )
		     setServerMiles ( TRUE );
		  else
		     setServerMiles ( FALSE );
	       }
	       else if ( serverNode.nodeName() == "Private" )
	       {
	          if ( serverNode.toElement().text() == "True" )
		     setServerPrivate ( TRUE );
		  else
		     setServerPrivate ( FALSE );
               }
	       else if ( serverNode.nodeName() == "Punkbuster" )
	       {
	          if ( serverNode.toElement().text() == "True" )
		     setServerPB ( TRUE );
		  else 
		     setServerPB ( FALSE );
	       }
	       else if ( serverNode.nodeName() == "Players" )
	       {
		  // Set number of players only when we have the data for them
	          setActualPlayerNumber();
	          QDomElement players = serverNode.toElement();
		  parsePlayers ( players );
	       }
	       else
	          setSettingValue ( serverNode.nodeName(),
		     serverNode.toElement().text());
	       serverNode = serverNode.nextSibling();
	    }
	 }
	 node = node.nextSibling();
      }
   }
}

void ServerListView::parsePlayers ( const QDomElement &parentElement )
{
   QDomNode node = parentElement.firstChild();
   int i = 0;
   while ( !node.isNull())
   {
      if ( node.isElement() && node.nodeName() == "Player" )
      {
         QDomNode playerValue = node.firstChild();
	 while ( !playerValue.isNull() && playerValue.isElement())
	 {
	    if ( playerValue.nodeName() == "Name" )
	    {
	       QString playerName = playerValue.toElement().text();
	       currentServer->setPlayerName ( i, playerName );
	       if ( buddyList->isBuddy ( playerName ))
	       {
	          currentServer->setPlayerBuddy ( i, TRUE );
	          setServerBuddyIcon ( TRUE );
	       }
	       else
	          currentServer->setPlayerBuddy ( i, FALSE );
	    }
	    else if ( playerValue.nodeName() == "Honor" )
	       currentServer->setPlayerHonor ( i, 
	          playerValue.toElement().text());
	    else if ( playerValue.nodeName() == "Goal" )
	       currentServer->setPlayerGoal ( i, playerValue.toElement().text());
	    else if ( playerValue.nodeName() == "Leader" )
	       currentServer->setPlayerLeader ( i,
	          playerValue.toElement().text());
	    else if ( playerValue.nodeName() == "Enemy" )
	       currentServer->setPlayerEnemy ( i,
	          playerValue.toElement().text());
	    else if ( playerValue.nodeName() == "Kia" )
	       currentServer->setPlayerKia ( i, playerValue.toElement().text());
	    else if ( playerValue.nodeName() == "Roe" )
	       currentServer->setPlayerRoe ( i, playerValue.toElement().text());
	    else if ( playerValue.nodeName() == "Ping" )
	       currentServer->setPlayerPing ( i, playerValue.toElement().text());
	    playerValue = playerValue.nextSibling();
	 }
      }
      node = node.nextSibling();
      i++;
   }
}
