// serverlistview.h
//
/****************************************************************************
   Copyright (C) 2004, David Henry. All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*****************************************************************************/

#ifndef SERVERLISTVIEW_H
#define SERVERLISTVIEW_H

#include <qsplitter.h>
#include <qdict.h>
#include "serverlistitem.h"

class BuddyList;
class QDomElement;
class QListView;
class QListViewItem;

class ServerListView: public QSplitter
{
   Q_OBJECT

public:
   ServerListView( QWidget *parent=0 );

   // Adding deleting
   void addNewServer ();
   bool addNewServer ( QString );
   void deleteCurrentServer ();
   void removeTimedoutServers();

   void setPlayerCount ( int );
   void setServerCount ( int );
   int getPlayerCount ();
   int getServerCount ();
   void setBuddyList ( BuddyList * );
   void findPlayer ( QString, bool, bool );
   void setTimeout ( QString );
   int getTimeoutNum ( QString );
   void setCurrentServer ( QString );
   bool isCurrentServerSet ();
   void selectCurrentServer ();

   // Setting current server values
   void setServerPing ( int );
   void setServerAddress ( QString );
   void setServerName ( QString );
   void setServerMaxPlayers ( QString );
   void setServerNumPlayers ( QString );
   void setServerMap ( QString );
   void setServerTour ( QString );
   void setServerMiles ( bool );
   void setServerOfficial ( bool );
   void setServerHonor ( bool );
   void setServerLeased ( bool );
   void setServerPrivate ( bool );
   void setServerPB ( bool );
   void setPlayerValue ( int, QString, QString );
   void setSettingValue ( QString, QString );
   void setPlayerBuddyIcon ( bool );
   void setServerBuddyIcon ( bool );
   void updatePlayersSettings ();

   bool showServer ( QString, QString );
   void setColouredNames ( bool );
   QStringList getServerAddressList ();
   QStringList getSelectedAddressList ();
   void clearServerList();
   void updateCurrentPing ( int );
   void clearPlayerSettingLists ();
   QString getCurrentName ();
   QString getCurrentAddress ();
   int getCurrentPlayers ();
   int getCurrentMaxPlayers ();
   QString getCurrentPlayerName ();
   bool isServerSelected ();
   void setServerVisible ( QString, QStringList, bool );
   void setServerVisible ( bool );
   void setActualPlayerNumber ();
   
   void saveServerData ( QString );
   void readServerData ( QString );

signals:
   void playerFound ( QString, QString, QString );
   void serverDoubleClicked ( QListViewItem *, const QPoint&, int );
   void playerRightClick ( QListViewItem *, const QPoint&, int );
   void serverRightClick ( QListViewItem *, const QPoint&, int );
   void serverClicked ( QListViewItem * );
   
protected slots:
   void clicked ( QListViewItem * );
   void doubleClicked ( QListViewItem *, const QPoint&, int );
   void rightClickPlayer ( QListViewItem *, const QPoint&, int );
   void rightClickServer ( QListViewItem *, const QPoint&, int );

private:
   QListView *serverList;
   QListView *playerList;
   QListView *settingsList;
   int playerCount;
   int serverCount;
   bool colouredNames;
   ServerListItem *currentServer;
   BuddyList *buddyList;
   void parsePlayers ( const QDomElement & );
   QDict<ServerListItem> list;
};

#endif
